@@ -85,7 +85,7 @@ parse_aarch64_dis_options (const char *options)
 {
   const char *option_end;
 
-  if (options == NULL)
+  if (!options)
     return;
 
   while (*options != '\0')
@@ -119,7 +119,7 @@ parse_aarch64_dis_options (const char *options)
    N.B. the fields are required to be in such an order than the most signficant
    field for VALUE comes the first, e.g. the <index> in
     SQDMLAL <Va><d>, <Vb><n>, <Vm>.<Ts>[<index>]
-   is encoded in H:L:M in some cases, the the fields H:L:M should be passed in
+   is encoded in H:L:M in some cases, the fields H:L:M should be passed in
    the order of H, L, M.  */
 
 static inline aarch64_insn
@@ -141,6 +141,7 @@ extract_fields (aarch64_insn code, aarch64_insn mask, ...)
       value <<= field->width;
       value |= extract_field (kind, code, mask);
     }
+  va_end (va);
   return value;
 }
 
@@ -1217,8 +1218,8 @@ decode_sizeq (aarch64_inst *inst)
   if (debug_dump)
     {
       int i;
-      for (i = 0; candidates[i] != AARCH64_OPND_QLF_NIL
-	   && i < AARCH64_MAX_QLF_SEQ_NUM; ++i)
+      for (i = 0; i < AARCH64_MAX_QLF_SEQ_NUM
+	   && candidates[i] != AARCH64_OPND_QLF_NIL; ++i)
 	DEBUG_TRACE ("qualifier %d: %s", i,
 		     aarch64_get_qualifier_name(candidates[i]));
       DEBUG_TRACE ("%d, %d", (int)value, (int)mask);
@@ -2118,6 +2119,7 @@ print_insn_aarch64_word (bfd_vma pc,
 			 uint32_t word,
 			 struct disassemble_info *info)
 {
+#if 0
   static const char *err_msg[6] =
     {
       [ERR_OK]   = "_",
@@ -2125,6 +2127,7 @@ print_insn_aarch64_word (bfd_vma pc,
       [-ERR_UNP] = "unpredictable",
       [-ERR_NYI] = "NYI"
     };
+#endif
 
   int ret;
   aarch64_inst inst;
@@ -2181,7 +2184,7 @@ aarch64_symbol_is_valid (asymbol * sym,
 {
   const char * name;
 
-  if (sym == NULL)
+  if (!sym)
     return FALSE;
 
   name = bfd_asymbol_name (sym);
@@ -2225,6 +2228,9 @@ get_sym_code_type (struct disassemble_info *info, int n,
   elf_symbol_type *es;
   unsigned int type;
   const char *name;
+  if (n < 0) {
+    return FALSE;
+  }
 
   es = *(elf_symbol_type **)(info->symtab + n);
   type = ELF_ST_TYPE (es->internal_elf_sym.st_info);
@@ -2300,7 +2306,7 @@ print_insn_aarch64 (bfd_vma pc,
 	  addr = bfd_asymbol_value (info->symtab[n]);
 	  if (addr > pc)
 	    break;
-	  if ((info->section == NULL
+	  if ((!info->section
 	       || info->section == info->symtab[n]->section)
 	      && get_sym_code_type (info, n, &type))
 	    {
@@ -2316,7 +2322,7 @@ print_insn_aarch64 (bfd_vma pc,
 	    n = last_mapping_sym;
 
 	  /* No mapping symbol found at this address.  Look backwards
-	     for a preceeding one.  */
+	     for a preceding one.  */
 	  for (; n >= 0; n--)
 	    {
 	      if (get_sym_code_type (info, n, &type))
