@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2013 - nibble */
+/* radare - LGPL - Copyright 2009-2014 - nibble */
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -11,36 +11,36 @@
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	ut8 buf2[4];
-	struct arm_insn *arminsn = arm_new();
+	struct winedbg_arm_insn *arminsn = arm_new();
 	arm_set_pc (arminsn, a->pc);
 	arm_set_thumb (arminsn, a->bits == 16);
 	if (a->big_endian && a->bits == 32) {
-		r_mem_copyendian (buf2, buf, 4, 0);
+		r_mem_swapendian (buf2, buf, 4);
 		arm_set_input_buffer (arminsn, buf2);
 	} else {
 		arm_set_input_buffer (arminsn, buf);
 	}
-	op->inst_len = arm_disasm_one_insn (arminsn);
-	strncpy (op->buf_asm, arm_insn_asm (arminsn), R_ASM_BUFSIZE);
-	strncpy (op->buf_hex, arm_insn_hex (arminsn), R_ASM_BUFSIZE);
+	op->size = arm_disasm_one_insn (arminsn);
+	strncpy (op->buf_asm, winedbg_arm_insn_asm (arminsn), R_ASM_BUFSIZE-1);
+	strncpy (op->buf_hex, winedbg_arm_insn_hex (arminsn), R_ASM_BUFSIZE-1);
 	arm_free (arminsn);
-	return op->inst_len;
+	return op->size;
 }
 
 RAsmPlugin r_asm_plugin_arm_winedbg = {
 	.name = "arm.winedbg",
 	.arch = "arm",
-	.bits = (int[]){ 16, 32, 0 },
-	.desc = "ARM disassembly plugin (winedbg backend)",
-	.init = NULL,
-	.fini = NULL,
+	.bits = 16|32,
+	.endian = R_SYS_ENDIAN_LITTLE | R_SYS_ENDIAN_BIG,
+	.desc = "WineDBG's ARM disassembler",
 	.disassemble = &disassemble,
-	.assemble = NULL
+	.license = "LGPL2"
 };
 
 #ifndef CORELIB
 struct r_lib_struct_t radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
-	.data = &r_asm_plugin_arm_winedbg
+	.data = &r_asm_plugin_arm_winedbg,
+	.version = R2_VERSION
 };
 #endif
