@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2012 - pancake */
+/* radare - LGPL - Copyright 2012-2014 - pancake */
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -18,23 +18,24 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 // XXX: This is wrong, some opcodes are 32bit in thumb mode
 static int assemble(RAsm *a, RAsmOp *op, const char *str) {
 	Bitbuf b = {.out = op->buf, .bits = 0};
-	return op->inst_len = rarvm_assemble (&b, str);
+	return op->size = rarvm_assemble (&b, str);
 }
 
 RAsmPlugin r_asm_plugin_rar = {
 	.name = "rar",
 	.arch = "rar",
-	.bits = (int[]){ 1, 0 },
-	.desc = "RAR VM disassembly plugin",
-	.init = NULL,
-	.fini = NULL,
+	.license = "LGPL3",
+	.bits = 1,
+	.endian = R_SYS_ENDIAN_NONE,
+	.desc = "RAR VM",
 	.disassemble = &disassemble,
-	.assemble = &assemble 
+	.assemble = &assemble
 };
 
 #ifndef CORELIB
 struct r_lib_struct_t radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
-	.data = &r_asm_plugin_rar
+	.data = &r_asm_plugin_rar,
+	.version = R2_VERSION
 };
 #endif
