@@ -6,32 +6,31 @@
 #include <r_asm.h>
 
 #include "../arch/z80/z80.c"
-
-static int do_assemble(RAsm *a, RAsmOp *op, const char *buf) {
-	return op->inst_len = z80asm (op->buf, buf);
-}
+#include "../arch/z80/z80asm.c"
 
 static int do_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	int dlen = z80dis (0, buf, op->buf_asm, len);
-	if (dlen<0) dlen = 0;
-	op->inst_len = dlen;
-	return op->inst_len;
+	return op->size = z80Disass (op, buf, len);
+}
+
+static int do_assemble(RAsm *a, RAsmOp *op, const char *buf) {
+	return op->size = z80asm (op->buf, buf);
 }
 
 RAsmPlugin r_asm_plugin_z80 = {
 	.name = "z80",
-	.desc = "z80 assembler plugin",
+	.desc = "Zilog Z80",
+	.license = "GPL",
 	.arch = "z80",
-	.bits = (int[]){ 8, 0 },
-	.init = NULL,
-	.fini = NULL,
-	.disassemble = do_disassemble,
-	.assemble = &do_assemble, 
+	.bits = 8,
+	.endian = R_SYS_ENDIAN_NONE,
+	.disassemble = &do_disassemble,
+	.assemble = &do_assemble,
 };
 
 #ifndef CORELIB
 struct r_lib_struct_t radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
-	.data = &r_asm_plugin_z80
+	.data = &r_asm_plugin_z80,
+	.version = R2_VERSION
 };
 #endif
