@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2010-2013 - nibble, pancake  */
+/* radare - LGPL - Copyright 2010-2016 - nibble, pancake  */
 
 #include <stdio.h>
 #include <r_types.h>
@@ -6,13 +6,13 @@
 #include "dyldcache.h"
 
 static int r_bin_dyldcache_init(struct r_bin_dyldcache_obj_t* bin) {
-	int len = r_buf_fread_at (bin->b, 0, (ut8*)&bin->hdr, "16c4il", 1);
+	int len = r_buf_fread_at (bin->b, 0, (ut8*)&bin->hdr, "16c4i7l", 1);
 	if (len == -1) {
 		perror ("read (cache_header)");
-		return R_FALSE;
+		return false;
 	}
 	bin->nlibs = bin->hdr.numlibs;
-	return R_TRUE;
+	return true;
 }
 
 static int r_bin_dyldcache_apply_patch (struct r_buf_t* buf, ut32 data, ut64 offset) {
@@ -23,14 +23,23 @@ static int r_bin_dyldcache_apply_patch (struct r_buf_t* buf, ut32 data, ut64 off
 
 /* TODO: Needs more testing and ERROR HANDLING */
 struct r_bin_dyldcache_lib_t *r_bin_dyldcache_extract(struct r_bin_dyldcache_obj_t* bin, int idx, int *nlib) {
+	ut64 liboff, linkedit_offset;
+	ut64 dyld_vmbase;
+	ut32 addend = 0;
 	struct r_bin_dyldcache_lib_t *ret = NULL;
+	struct dyld_cache_image_info* image_infos = NULL;
 	struct mach_header *mh;
-	RBuffer* dbuf;
-	ut64 curoffset, liboff, libla, libpath, linkedit_offset;
 	ut8 *data, *cmdptr;
-	char *libname;
 	int cmd, libsz = 0;
+	RBuffer* dbuf;
+	char *libname;
 
+	if (!bin)
+	    	return NULL;
+	if (bin->size < 1) {
+		eprintf ("Empty file? (%s)\n", bin->file? bin->file: "(null)");
+		return NULL;
+	}
 	if (bin->nlibs < 0 || idx < 0 || idx > bin->nlibs)
 		return NULL;
 	*nlib = bin->nlibs;
@@ -39,21 +48,38 @@ struct r_bin_dyldcache_lib_t *r_bin_dyldcache_extract(struct r_bin_dyldcache_obj
 		perror ("malloc (ret)");
 		return NULL;
 	}
-	curoffset = bin->hdr.startaddr+idx*32;
-	libla = *(ut64*)(bin->b->buf+curoffset);
-	liboff = libla - *(ut64*)&bin->b->buf[bin->hdr.baseaddroff];
-	if (liboff < 0 || liboff > bin->size) {
+	if (bin->hdr.startaddr > bin->size) {
+	    	eprintf ("corrupted dyldcache");
+		free (ret);
+		return NULL;
+	}
+	if (bin->hdr.startaddr > bin->size || bin->hdr.baseaddroff > bin->size) {
+		eprintf ("corrupted dyldcache");
+		free (ret);
+		return NULL;
+	}
+	image_infos = (struct dyld_cache_image_info*) (bin->b->buf + bin->hdr.startaddr);
+	dyld_vmbase = *(ut64 *)(bin->b->buf + bin->hdr.baseaddroff);
+	liboff = image_infos[idx].address - dyld_vmbase;
+	if (liboff > bin->size) {
 		eprintf ("Corrupted file\n");
 		free (ret);
 		return NULL;
 	}
 	ret->offset = liboff;
-	libpath = *(ut64*)(bin->b->buf+curoffset + 24);
+	if (image_infos[idx].pathFileOffset > bin->size) {
+	    eprintf ("corrupted file\n");
+		free (ret);
+		return NULL;
+	}
+	libname = (char *)(bin->b->buf + image_infos[idx].pathFileOffset);
 	/* Locate lib hdr in cache */
-	data = bin->b->buf+liboff;
+	data = bin->b->buf + liboff;
 	mh = (struct mach_header *)data;
 	/* Check it is mach-o */
-	if (mh->magic != 0xfeedface) {
+	if (mh->magic != MH_MAGIC && mh->magic != MH_MAGIC_64) {
+	    	if (mh->magic == 0xbebafeca) //FAT binary
+		    	eprintf ("FAT Binary\n");
 		eprintf ("Not mach-o\n");
 		free (ret);
 		return NULL;
@@ -64,16 +90,18 @@ struct r_bin_dyldcache_lib_t *r_bin_dyldcache_extract(struct r_bin_dyldcache_obj
 		free (ret);
 		return NULL;
 	}
-	r_buf_set_bytes (dbuf, data, sizeof (struct mach_header));
-	cmdptr = data + sizeof(struct mach_header);
+	addend = mh->magic == MH_MAGIC? sizeof (struct mach_header) : sizeof (struct mach_header_64);
+	r_buf_set_bytes (dbuf, data, addend);
+	cmdptr = data + addend;
 	/* Write load commands */
 	for (cmd = 0; cmd < mh->ncmds; cmd++) {
 		struct load_command *lc = (struct load_command *)cmdptr;
-		cmdptr += lc->cmdsize;
 		r_buf_append_bytes (dbuf, (ut8*)lc, lc->cmdsize);
+		cmdptr += lc->cmdsize;
 	}
+	cmdptr = data + addend;
 	/* Write segments */
-	for (cmd = linkedit_offset = 0, cmdptr = data + sizeof (struct mach_header); cmd < mh->ncmds; cmd++) {
+	for (cmd = linkedit_offset = 0; cmd < mh->ncmds; cmd++) {
 		struct load_command *lc = (struct load_command *)cmdptr;
 		cmdptr += lc->cmdsize;
 		switch (lc->cmd) {
@@ -82,11 +110,14 @@ struct r_bin_dyldcache_lib_t *r_bin_dyldcache_extract(struct r_bin_dyldcache_obj
 			/* Write segment and patch offset */
 			struct segment_command *seg = (struct segment_command *)lc;
 			int t = seg->filesize;
-			if (seg->fileoff+seg->filesize > bin->b->length)
-				t = bin->b->length - seg->fileoff;
+			if (seg->fileoff + seg->filesize > bin->size || seg->fileoff > bin->size) {
+				eprintf ("malformed dyldcache\n");
+				free (ret);
+				r_buf_free (dbuf);
+				return NULL;
+			}
 			r_buf_append_bytes (dbuf, bin->b->buf+seg->fileoff, t);
-			r_bin_dyldcache_apply_patch (dbuf, dbuf->length,
-				(ut64)((size_t)&seg->fileoff - (size_t)data));
+			r_bin_dyldcache_apply_patch (dbuf, dbuf->length, (ut64)((size_t)&seg->fileoff - (size_t)data));
 			/* Patch section offsets */
 			int sect_offset = seg->fileoff - libsz;
 			libsz = dbuf->length;
@@ -137,35 +168,67 @@ struct r_bin_dyldcache_lib_t *r_bin_dyldcache_extract(struct r_bin_dyldcache_obj
 	}
 	/* Fill r_bin_dyldcache_lib_t ret */
 	ret->b = dbuf;
-	libname = (char*)(bin->b->buf+libpath);
-	strncpy (ret->path, libname, sizeof (ret->path)-1);
+	strncpy (ret->path, libname, sizeof (ret->path) - 1);
 	ret->size = libsz;
 	return ret;
 }
 
 void* r_bin_dyldcache_free(struct r_bin_dyldcache_obj_t* bin) {
-	if (!bin)
+	if (!bin) {
 		return NULL;
-	if (bin->b)
-		r_buf_free (bin->b);
-	free(bin);
+	}
+	r_buf_free (bin->b);
+	free (bin);
 	return NULL;
 }
 
+void r_bin_dydlcache_get_libname(struct r_bin_dyldcache_lib_t *lib, char **libname) {
+	char *cur = lib->path;
+	char *res = lib->path;
+	int path_length = strlen (lib->path);
+	while (cur < cur + path_length - 1) {
+		cur = strchr (cur, '/');
+		if (!cur) {
+			break;
+		}
+		cur++;
+		res = cur;
+	}
+	*libname = res;
+}
+
 struct r_bin_dyldcache_obj_t* r_bin_dyldcache_new(const char* file) {
 	struct r_bin_dyldcache_obj_t *bin;
 	ut8 *buf;
+	if (!(bin = R_NEW0 (struct r_bin_dyldcache_obj_t)))
+		return NULL;
+	bin->file = file;
+	if (!(buf = (ut8*)r_file_slurp (file, &bin->size))) {
+		return r_bin_dyldcache_free (bin);
+	}
+	bin->b = r_buf_new ();
+	if (!r_buf_set_bytes (bin->b, buf, bin->size)) {
+		free (buf);
+		return r_bin_dyldcache_free (bin);
+	}
+	free (buf);
+	if (!r_bin_dyldcache_init (bin))
+		return r_bin_dyldcache_free (bin);
+	return bin;
+}
+
+struct r_bin_dyldcache_obj_t* r_bin_dyldcache_from_bytes_new(const ut8* buf, ut64 size) {
+	struct r_bin_dyldcache_obj_t *bin;
 	if (!(bin = malloc (sizeof (struct r_bin_dyldcache_obj_t))))
 		return NULL;
 	memset (bin, 0, sizeof (struct r_bin_dyldcache_obj_t));
-	bin->file = file;
-	if (!(buf = (ut8*)r_file_slurp(file, &bin->size))) 
-		return r_bin_dyldcache_free(bin);
+	if (!buf)
+		return r_bin_dyldcache_free (bin);
 	bin->b = r_buf_new();
-	if (!r_buf_set_bytes(bin->b, buf, bin->size))
-		return r_bin_dyldcache_free(bin);
-	free (buf);
-	if (!r_bin_dyldcache_init(bin))
-		return r_bin_dyldcache_free(bin);
+	if (!r_buf_set_bytes (bin->b, buf, size))
+		return r_bin_dyldcache_free (bin);
+	if (!r_bin_dyldcache_init (bin))
+		return r_bin_dyldcache_free (bin);
+	bin->size = size;
 	return bin;
 }
