@@ -1,12 +1,12 @@
-/* radare - LGPL - Copyright 2009 pancake<nopcode.org> */
+/* radare - LGPL - Copyright 2009-2015 - pancake */
 
 #include <r_bp.h>
 #include <r_lib.h>
 
 static struct r_bp_arch_t r_bp_plugin_sh_bps[] = {
-	{ 2, 0, (const ut8*)"\x20\xc3" }, //Little endian bp
-	{ 2, 1, (const ut8*)"\xc3\x20" }, //Big endian bp
-	{ 0, 0, NULL },
+	{ 32, 2, 0, (const ut8*)"\x20\xc3" }, //Little endian bp
+	{ 32, 2, 1, (const ut8*)"\xc3\x20" }, //Big endian bp
+	{ 0, 0, 0, NULL },
 };
 
 struct r_bp_plugin_t r_bp_plugin_sh = {
@@ -20,5 +20,6 @@ struct r_bp_plugin_t r_bp_plugin_sh = {
 struct r_lib_struct_t radare_plugin = {
 	.type = R_LIB_TYPE_BP,
 	.data = &r_bp_plugin_sh,
+	.version = R2_VERSION
 };
 #endif
