@@ -1,4 +1,26 @@
-/* radare - LGPL - Copyright 2009-2012 // pancake<nopcode.org> */
+/* radare - LGPL - Copyright 2009-2015 // pancake */
+
+static int cmd_mkdir(void *data, const char *input) {
+	r_core_syscmd_mkdir (input);
+	return 0;
+}
+
+static int cmd_mv(void *data, const char *input) {
+	if (strlen (input)<3) {
+		eprintf ("Usage: mv src dst\n");
+		return 0;
+	}
+	input = input + 2;
+	if (!r_sandbox_enable(0)) {
+#if __WINDOWS__
+		r_sys_cmdf ("move %s", input);
+#else
+		r_sys_cmdf ("mv %s", input);
+#endif
+	}
+	return 0;
+}
+
 static int cmd_mount(void *data, const char *_input) {
 	ut64 off = 0;
 	char *input, *oinput, *ptr, *ptr2;
@@ -9,6 +31,13 @@ static int cmd_mount(void *data, const char *_input) {
 	RFSPlugin *plug;
 	RFSPartition *part;
 	RCore *core = (RCore *)data;
+
+	if (!strncmp ("kdir", _input, 4)) {
+		return cmd_mkdir (data, _input);
+	}
+	if (!strncmp ("v", _input, 1)) {
+		return cmd_mv (data, _input);
+	}
 	input = oinput = strdup (_input);
 
 	switch (*input) {
@@ -139,7 +168,7 @@ static int cmd_mount(void *data, const char *_input) {
 					r_str_chop_path (ptr);
 					printf ("%s\n", ptr);
 				}
-				//XXX: r_list_destroy (list);
+				//XXX: r_list_purge (list);
 			} else eprintf ("Unknown store path\n");
 			break;
 		case 'o':
@@ -155,12 +184,16 @@ static int cmd_mount(void *data, const char *_input) {
 					r_str_chop_path (ptr);
 					printf ("%s\n", ptr);
 				}
-				//XXX: r_list_destroy (list);
+				//XXX: r_list_purge (list);
 			} else eprintf ("Unknown store path\n");
 			break;
 		}
 		break;
 	case 's':
+		if (core->http_up) {
+			free (oinput);
+			return false;
+		}
 		input++;
 		if (input[0]==' ')
 			input++;
@@ -169,26 +202,27 @@ static int cmd_mount(void *data, const char *_input) {
 	case 'y':
 		eprintf ("TODO\n");
 		break;
-	case '?':
-		r_cons_printf (
-		"Usage: m[-?*dgy] [...]\n"
-		" m              ; list all mountpoints in human readable format\n"
-		" m*             ; same as above, but in r2 commands\n"
-		" ml             ; list filesystem plugins\n"
-		" m /mnt         ; mount fs at /mnt with autodetect fs and current offset\n"
-		" m /mnt ext2 0  ; mount ext2 fs at /mnt with delta 0 on IO\n"
-		" m-/            ; umount given path (/)\n"
-		" my             ; yank contents of file into clipboard\n"
-		" mo /foo        ; get offset and size of given file\n"
-		" mg /foo        ; get contents of file/dir dumped to disk (XXX?)\n"
-		" mf[o|n]        ; search files for given filename or for offset\n"
-		" md /           ; list directory contents for path\n"
-		" mp             ; list all supported partition types\n"
-		" mp msdos 0     ; show partitions in msdos format at offset 0\n"
-		" ms /mnt        ; open filesystem prompt at /mnt\n"
-		" m?             ; show this help\n"
-		"TODO: support multiple mountpoints and RFile IO's (need io+core refactor)\n"
-		);
+	case '?': {
+		const char* help_msg[] = {
+			"Usage:", "m[-?*dgy] [...] ", "Mountpoints management",
+			"m", "", "List all mountpoints in human readable format",
+			"m*", "", "Same as above, but in r2 commands",
+			"ml", "", "List filesystem plugins",
+			"m", " /mnt", "Mount fs at /mnt with autodetect fs and current offset",
+			"m", " /mnt ext2 0", "Mount ext2 fs at /mnt with delta 0 on IO",
+			"m-/", "", "Umount given path (/)",
+			"my", "", "Yank contents of file into clipboard",
+			"mo", " /foo", "Get offset and size of given file",
+			"mg", " /foo", "Get contents of file/dir dumped to disk (XXX?)",
+			"mf", "[?] [o|n]", "Search files for given filename or for offset",
+			"md", " /", "List directory contents for path",
+			"mp", "", "List all supported partition types",
+			"mp", " msdos 0", "Show partitions in msdos format at offset 0",
+			"ms", " /mnt", "Open filesystem prompt at /mnt",
+			//"TODO: support multiple mountpoints and RFile IO's (need io+core refactorn",
+			NULL};
+		r_core_cmd_help (core, help_msg);
+		}
 		break;
 	}
 	free (oinput);
