@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2010-2013 - pancake */
+/* radare - LGPL - Copyright 2010-2015 - pancake */
 
 #include <r_egg.h>
 
@@ -94,7 +94,7 @@ static int oc = '\n';
 static int mode = NORMAL;
 
 static char *find_include(const char *prefix, const char *file) {
-	char *pfx, *ret, *env = r_sys_getenv (R_EGG_INCDIR_ENV);
+	char *pfx = NULL, *ret = NULL, *env = r_sys_getenv (R_EGG_INCDIR_ENV);
 	//eprintf ("find_include (%s,%s)\n", prefix, file);
 	if (!prefix) prefix = "";
 	if (*prefix=='$') {
@@ -102,7 +102,10 @@ static char *find_include(const char *prefix, const char *file) {
 		pfx = out? out: strdup ("");
 	} else {
 		pfx = strdup (prefix);
-		if (!pfx) return NULL;
+		if (!pfx) {
+			free (env);
+			return NULL;
+		}
 	}
 
 	if (env) {
@@ -112,6 +115,7 @@ static char *find_include(const char *prefix, const char *file) {
 		while (str) {
 			if (ptr)
 				*ptr = 0;
+			free (ret);
 			ret = r_str_concatf (NULL, "%s/%s", pfx, file);
 			{
 				char *filepath = r_str_concatf (NULL, "%s/%s/%s", str, pfx, file);
@@ -119,6 +123,7 @@ static char *find_include(const char *prefix, const char *file) {
 				if (r_file_exists (filepath)) {
 					free (env);
 					free (pfx);
+					free (ret);
 					return filepath;
 				}
 				free (filepath);
@@ -134,12 +139,16 @@ static char *find_include(const char *prefix, const char *file) {
 }
 
 R_API void r_egg_lang_include_path (REgg *egg, const char *path) {
+	char *tmp_ptr = NULL;
 	char *env = r_sys_getenv (R_EGG_INCDIR_ENV);
 	if (!env || !*env) {
 		r_egg_lang_include_init (egg);
+		free (env);
 		env = r_sys_getenv (R_EGG_INCDIR_ENV);
+		tmp_ptr = env;
 	}
 	env = r_str_concatf (NULL, "%s:%s", path, env);
+	free (tmp_ptr);
 	r_sys_setenv (R_EGG_INCDIR_ENV, env);
 	free (env);
 }
@@ -197,7 +206,7 @@ static char *get_end_frame_label(REgg *egg) {
 #endif
 
 static void rcc_pusharg(REgg *egg, char *str) {
-	REggEmit *e = egg->emit;
+	REggEmit *e = egg->remit;
 	char buf[64], *p = r_egg_mkvar (egg, buf, str, 0);
 	if (!p) return;
 	// TODO: free (ctxpush[context]);
@@ -206,10 +215,11 @@ static void rcc_pusharg(REgg *egg, char *str) {
 	if (pushargs)
 		e->push_arg (egg, varxs, nargs, p);
 	//ctxpush[context+nbrackets] = strdup(str); // use nargs??? (in callname)
+	free (p);
 }
 
 static void rcc_element(REgg *egg, char *str) {
-	REggEmit *e = egg->emit;
+	REggEmit *e = egg->remit;
 	char *p = strrchr (str, ',');
 	int num, num2;
 
@@ -228,7 +238,7 @@ static void rcc_element(REgg *egg, char *str) {
 		else
 		if (mode == NORMAL) {
 			if (!atoi (str)) {
-				if (dstvar == NULL) /* return string */
+				if (!dstvar) /* return string */
 					dstvar = strdup (".fix0");
 				rcc_pushstr (egg, str, 1);
 			}
@@ -241,6 +251,11 @@ static void rcc_element(REgg *egg, char *str) {
 			mode = NORMAL;
 			break;
 		case SYSCALL:
+			if (nsyscalls > 255){
+				eprintf ("global-buffer-overflow in syscalls\n");
+				break;
+			}
+			//XXX the mem for name and arg are not freed - MEMLEAK
 			syscalls[nsyscalls].name = strdup (dstvar);
 			syscalls[nsyscalls].arg = strdup (str);
 			nsyscalls++;
@@ -254,7 +269,7 @@ static void rcc_element(REgg *egg, char *str) {
 			p = strchr (str, ',');
 			if (p) {
 				*p='\0';
-				num2 = atoi (p+1); 
+				num2 = atoi (p+1);
 			} else num2 = 0;
 			num = atoi (str) + num2;
 			stackframe = num;
@@ -270,7 +285,7 @@ static void rcc_element(REgg *egg, char *str) {
 static void rcc_pushstr(REgg *egg, char *str, int filter) {
         int dotrim = 1;
         int i, j, len;
-	REggEmit *e = egg->emit;
+	REggEmit *e = egg->remit;
 
         e->comment (egg, "encode %s string (%s) (%s)",
                 filter? "filtered": "unfiltered", str, callname);
@@ -298,42 +313,41 @@ static void rcc_pushstr(REgg *egg, char *str, int filter) {
 
 R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 	int i, idx, len, qi;
-	char *str, foo[32], *q, *ret;
+	char *oldstr = NULL, *str = NULL, foo[32], *q, *ret = NULL;
 
 	delta += stackfixed; // XXX can be problematic
-	if (_str == NULL)
+	if (!_str)
 		return NULL; /* fix segfault, but not badparsing */
 	/* XXX memory leak */
- 	ret = str = strdup (skipspaces (_str));
+	ret = str = oldstr = strdup (skipspaces (_str));
 	//if (num || str[0]=='0') { sprintf(out, "$%d", num); ret = out; }
 	if ( (q = strchr (str, ':')) ) {
 		*q = '\0';
 		qi = atoi (q+1);
 		varsize = (qi==1)? 'b': 'l';
 	} else varsize='l';
-	if (*str=='*'||*str=='&') {
+	if (*str == '*' || *str == '&') {
 		varxs = *str;
 		str++;
 	} else varxs = 0;
-	if (str[0]=='.') {
-		REggEmit *e = egg->emit;
-		ret = out;
+	if (str[0] == '.') {
+		REggEmit *e = egg->remit;
 		idx = atoi (str+4) + delta + e->size;
-		if (!memcmp (str+1, "ret", 3)) {
+		if (!strncmp (str+1, "ret", 3)) {
 			strcpy (out, e->retvar);
 		} else
-		if (!memcmp (str+1, "fix", 3)) {
+		if (!strncmp (str+1, "fix", 3)) {
 			e->get_var (egg, 0, out, idx-stackfixed);
 			//sprintf(out, "%d(%%"R_BP")", -(atoi(str+4)+delta+R_SZ-stackfixed));
 		} else
-		if (!memcmp (str+1, "var", 3)) {
+		if (!strncmp (str+1, "var", 3)) {
 			e->get_var (egg, 0, out, idx);
 		//sprintf(out, "%d(%%"R_BP")", -(atoi(str+4)+delta+R_SZ));
 		} else
-		if (!memcmp (str+1, "arg", 3)) {
+		if (!strncmp (str+1, "arg", 3)) {
 			if (str[4]) {
 				if (stackframe == 0) {
-					e->get_var (egg, 1, out, 4); //idx-4); 
+					e->get_var (egg, 1, out, 4); //idx-4);
 				} else {
 					e->get_var (egg, 2, out, idx+4);
 				}
@@ -341,21 +355,25 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 				/* TODO: return size of syscall */
 				if (callname) {
 					for (i=0; i<nsyscalls; i++)
-						if (!strcmp (syscalls[i].name, callname))
-							return syscalls[i].arg;
+						if (!strcmp (syscalls[i].name, callname)) {
+							free (oldstr);
+							return strdup(syscalls[i].arg);
+						}
 					eprintf ("Unknown arg for syscall '%s'\n", callname);
 				} else eprintf ("NO CALLNAME '%s'\n", callname);
 			}
 		} else
-		if (!memcmp (str+1, "reg", 3)) {
+		if (!strncmp (str+1, "reg", 3)) {
 			// XXX: can overflow if out is small
 			if (attsyntax)
 				snprintf (out, 32, "%%%s", e->regs (egg, atoi (str+4)));
 			else snprintf (out, 32, "%s", e->regs (egg, atoi (str+4)));
 		} else {
-			ret = str; /* TODO: show error, invalid var name? */
-			eprintf ("FUCKED UP\n");
+			out = str; /* TODO: show error, invalid var name? */
+			eprintf ("Something is really wrong\n");
 		}
+		ret = strdup(out);
+		free (oldstr);
 	} else if (*str=='"' || *str=='\'') {
 		int mustfilter = *str=='"';
 		/* TODO: check for room in stackfixed area */
@@ -366,17 +384,18 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 				stackfixed, len);
 		str[len]='\0';
 		snprintf (foo, sizeof (foo)-1, ".fix%d", nargs*16); /* XXX FIX DELTA !!!1 */
+		free(dstvar);
 		dstvar = strdup (skipspaces (foo));
 		rcc_pushstr (egg, str, mustfilter);
 		ret = r_egg_mkvar (egg, out, foo, 0);
+		free (oldstr);
 	}
-	//free ((void *)_str);
 	return ret;
 }
 
 static void rcc_fun(REgg *egg, const char *str) {
 	char *ptr, *ptr2;
-	REggEmit *e = egg->emit;
+	REggEmit *e = egg->remit;
 	str = skipspaces (str);
 	if (CTX) {
 		ptr = strchr (str, '=');
@@ -390,7 +409,7 @@ static void rcc_fun(REgg *egg, const char *str) {
 		} else {
 			str = skipspaces (str);
 			rcc_set_callname (skipspaces (str));
-			egg->emit->comment (egg, "rcc_fun %d (%s)",
+			egg->remit->comment (egg, "rcc_fun %d (%s)",
 				CTX, callname);
 		}
 	} else {
@@ -502,10 +521,14 @@ static void set_nested(REgg *egg, const char *s) {
 }
 
 static void rcc_context(REgg *egg, int delta) {
-	REggEmit *emit = egg->emit;
+	REggEmit *emit = egg->remit;
 	char str[64];
 
-	nestedi[CTX-1]++;
+
+	if (CTX>31 || CTX <0)
+		return;
+
+	nestedi[CTX]++;
 	if (callname && CTX>0) {// && delta>0) {
 	//	set_nested (callname);
 //eprintf (" - - - - - - -  set nested d=%d c=%d (%s)\n", delta, context-1, callname);
@@ -515,8 +538,8 @@ static void rcc_context(REgg *egg, int delta) {
 	lastctxdelta = delta;
 
 	if (CTX == 0 && delta < 0) {
-if (mode != NAKED)
-		emit->frame_end (egg, stackframe+stackfixed, nbrackets);
+		if (mode != NAKED)
+			emit->frame_end (egg, stackframe+stackfixed, nbrackets);
 		if (mode == NORMAL) /* XXX : commenting this makes hello.r unhappy! TODO: find a cleaner alternative */
 			stackframe = 0;
 		mode = NORMAL;
@@ -581,7 +604,7 @@ emit->while_end (egg, get_frame_label (context-1));
 				emit->branch (egg, b, g, e, n, varsize, str);
 				if (CTX>0) {
 					/* XXX .. */
-				} else eprintf ("FUCKING CASE\n");
+				}
 				rcc_reset_callname ();
 			} //else eprintf ("Unknown statement (%s)(%s)\n", cn, elem);
 		} // handle '{ ..'
@@ -593,6 +616,9 @@ static int parsedatachar(REgg *egg, char c) {
 	char *str;
 	int i, j;
 
+	if (!dstval) {
+		return 0;
+	}
 	/* skip until '{' */
 	if (c == '{') { /* XXX: repeated code!! */
 		rcc_context (egg, 1);
@@ -602,14 +628,15 @@ static int parsedatachar(REgg *egg, char c) {
 		/* capture value between parenthesis foo@data(NNN) { ... } */
 		if (c==')') {
 			stackframe = atoi (dstval);
-eprintf ("STACKTRAF %d\n", stackframe);
 			ndstval = 0;
-		} else dstval[ndstval++] = c;
+		} else {
+			dstval[ndstval++] = c;
+		}
 		return 0;
 	}
 	/* capture body */
 	if (c == '}') { /* XXX: repeated code!! */
-		if (CTX< 1) {
+		if (CTX< 2) {
 			inlinectr = 0;
 			rcc_context (egg, -1);
 			slurp = 0;
@@ -617,7 +644,7 @@ eprintf ("STACKTRAF %d\n", stackframe);
 			/* register */
 			if (dstval != NULL && dstvar != NULL) {
 				dstval[ndstval]='\0';
-				egg->emit->comment (egg, "data (%s)(%s)size=(%d)\n",
+				egg->remit->comment (egg, "data (%s)(%s)size=(%d)\n",
 					dstvar, dstval, stackframe);
 				r_egg_printf (egg, ".data\n");
 				for (str=dstval; is_space (*str); str++);
@@ -635,7 +662,7 @@ eprintf ("STACKTRAF %d\n", stackframe);
 				ndstval = 0;
 				CTX = 0;
 				return 1;
-			} else eprintf ("FUCK FUCK\n");
+			}
 		}
 	}
 	dstval[ndstval++] = c;
@@ -662,7 +689,7 @@ static int parseinlinechar(REgg *egg, char c) {
 			slurp = 0;
 			mode = NORMAL;
 			inlinectr = 0;
-			if (dstvar == NULL && dstval == syscallbody) {
+			if (!dstvar && dstval == syscallbody) {
 				dstval = NULL;
 				return 1;
 			} else
@@ -676,7 +703,8 @@ static int parseinlinechar(REgg *egg, char c) {
 				R_FREE (dstvar);
 				R_FREE (dstval);
 				return 1;
-			} else eprintf ("FUCK FUCK\n");
+			}
+			eprintf ("Parse error\n");
 		}
 	}
 	dstval[ndstval++] = c;
@@ -687,12 +715,12 @@ static int parseinlinechar(REgg *egg, char c) {
 /* TODO: split this function into several ones..quite long fun */
 static void rcc_next(REgg *egg) {
 	const char *ocn;
-	REggEmit *e = egg->emit;
-	char *str, *p, *ptr, buf[64];
+	REggEmit *e = egg->remit;
+	char *str = NULL, *p, *ptr, buf[64];
 	int i;
 
 	if (setenviron) {
-		elem[elem_n-1] = 0;
+		elem[elem_n - 1] = 0;
 		r_sys_setenv (setenviron, elem);
 		R_FREE (setenviron);
 		return;
@@ -719,7 +747,9 @@ static void rcc_next(REgg *egg) {
 				r_egg_lang_parsechar (egg, *p);
 			free (q);
 			line = oline;
-		} else eprintf ("Cannot find '%s'\n", path);
+		} else {
+			eprintf ("Cannot find '%s'\n", path);
+		}
 		free (path);
 		return;
 	}
@@ -729,7 +759,7 @@ static void rcc_next(REgg *egg) {
 			if (nargs != 1) {
 				eprintf ("Invalid number of arguments for goto()\n");
 				return;
-			} 
+			}
 			e->jmp (egg, ctxpush[CTX], 0);
 			rcc_reset_callname ();
 			return;
@@ -742,16 +772,20 @@ static void rcc_next(REgg *egg) {
 		ptr = strchr (callname, '=');
 		if (ptr) {
 			*ptr = '\0';
-			ocn = ptr+1;
+			//ocn = ptr+1; // what is the point of this?
 		}
 		ocn = skipspaces (callname);
+		if (!ocn) return;
 		str = r_egg_mkvar (egg, buf, ocn, 0);
+		if (!str) {
+			eprintf ("Cannot mkvar\n");
+			return;
+		}
 		if (*ocn=='.')
 			e->call (egg, str, 1);
-		else
 		if (!strcmp (str, "while")) {
 			char var[128];
-			if (lastctxdelta>=0)
+			if (lastctxdelta >= 0)
 				exit (eprintf ("ERROR: Unsupported while syntax\n"));
 			sprintf (var, "__begin_%d_%d_%d\n", nfunctions, CTX, nestedi[CTX-1]);
 			e->while_end (egg, var); //get_frame_label (1));
@@ -773,7 +807,7 @@ static void rcc_next(REgg *egg) {
 #endif
 			nargs = 0;
 		} else {
-			for (i=0; i<nsyscalls; i++) {
+			for (i = 0; i < nsyscalls; i++) {
 				if (!strcmp (str, syscalls[i].name)) {
 					p = syscallbody;
 					e->comment (egg, "set syscall args");
@@ -790,14 +824,14 @@ static void rcc_next(REgg *egg) {
 							for (q=s; *q; q++)
 								r_egg_lang_parsechar (egg, *q);
 							free (s);
-						} else eprintf ("Cant get @syscall payload\n");
+						} else eprintf ("Cannot get @syscall payload\n");
 					}
 					docall = 0;
 					break;
 				}
 			}
 			if (docall)
-			for (i=0; i<ninlines; i++) {
+			for (i = 0; i < ninlines; i++) {
 				if (!strcmp (str, inlines[i].name)) {
 					p = inlines[i].body;
 					docall = 0;
@@ -814,11 +848,12 @@ static void rcc_next(REgg *egg) {
 				e->call (egg, str, 0);
 			}
 		}
-		if (nargs>0)
+		if (nargs > 0)
 			e->restore_stack (egg, nargs*e->size);
 		if (ocn) { // Used to call .var0()
 			/* XXX: Probably buggy and wrong */
 			*buf = 0;
+			free (str);
 			str = r_egg_mkvar (egg, buf, ocn, 0);
 			if (*buf)
 				e->get_result (egg, buf);
@@ -826,13 +861,14 @@ static void rcc_next(REgg *egg) {
 		}
 		/* store result of call */
 		if (dstvar) {
-if (mode != NAKED) {
-			*buf = 0;
-			str = r_egg_mkvar (egg, buf, dstvar, 0);
-			if (*buf == 0)
-				eprintf ("Cannot resolve variable '%s'\n", dstvar);
-			else e->get_result (egg, buf);
-}
+			if (mode != NAKED) {
+				*buf = 0;
+				free (str);
+				str = r_egg_mkvar (egg, buf, dstvar, 0);
+				if (*buf == 0)
+					eprintf ("Cannot resolve variable '%s'\n", dstvar);
+				else e->get_result (egg, buf);
+			}
 			R_FREE (dstvar);
 		}
 		rcc_reset_callname ();
@@ -857,11 +893,15 @@ if (mode != NAKED) {
 					/* XXX this is a hack .. must be integrated with pusharg */
 					if (varxs=='&')
 						e->load_ptr (egg, eq);
-					eq = NULL;
+					if (eq) {
+						free (eq);
+						eq = NULL;
+					}
 					type = ' ';
 				} else type = '$';
 				vs = 'l'; // XXX: add support for != 'l' size
 				e->mathop (egg, ch, vs, type, eq, p);
+				free(p);
 			} else {
 				if (!strcmp (ptr, "break")) { // handle 'break;'
 					e->trap (egg);
@@ -872,11 +912,12 @@ if (mode != NAKED) {
 			}
 		}
 	}
+	free (str);
 }
 
 R_API int r_egg_lang_parsechar(REgg *egg, char c) {
-	REggEmit *e = egg->emit;
-	char *ptr, str[64];
+	REggEmit *e = egg->remit;
+	char *ptr, str[64], *tmp_ptr = NULL;
 	if (c=='\n') {
 		line++;
 		elem_n = 0;
@@ -888,7 +929,7 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 			oc = c;
 			return 0;
 		}
-		skipline = 0; 
+		skipline = 0;
 	}
 	if (mode == DATA)
 		return parsedatachar (egg, c);
@@ -901,7 +942,9 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 				if (c == '`') {
 					elem[elem_n] = 0;
 					elem_n = 0;
-					r_egg_printf (egg, "%s", r_egg_mkvar (egg, str, elem, 0));
+					tmp_ptr = r_egg_mkvar (egg, str, elem, 0);
+					r_egg_printf (egg, "%s", tmp_ptr);
+					free (tmp_ptr);
 					quotelinevar = 0;
 				} else elem[elem_n++] = c;
 			} else {
