@@ -37,6 +37,8 @@
 #include "r_hash.h"
 #include "sha2.h"
 
+#define WEAK_ALIASING 0
+
 /*
  * ASSERT NOTE:
  * Some sanity checking code is included using assert().  On my FreeBSD
@@ -613,7 +615,14 @@ void SHA256_Final(sha2_byte digest[], R_SHA256_CTX* context) {
 			*context->buffer = 0x80;
 		}
 		/* Set the bit count: */
+#if WEAK_ALIASING
 		*(sha2_word64*)&context->buffer[SHA256_SHORT_BLOCK_LENGTH] = context->bitcount;
+#else
+		{
+			sha2_word64 *p = (sha2_word64*)((ut8*)context->buffer+SHA256_SHORT_BLOCK_LENGTH);
+			*p = (sha2_word64)context->bitcount;
+		}
+#endif
 
 		/* Final transform: */
 		SHA256_Transform(context, (sha2_word32*)context->buffer);
@@ -633,7 +642,7 @@ void SHA256_Final(sha2_byte digest[], R_SHA256_CTX* context) {
 	}
 
 	/* Clean up state data: */
-	MEMSET_BZERO(context, sizeof(context));
+	MEMSET_BZERO(context, sizeof(*context));
 	usedspace = 0;
 }
 
@@ -654,7 +663,7 @@ char *SHA256_End(R_SHA256_CTX* context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-		MEMSET_BZERO(context, sizeof(context));
+		MEMSET_BZERO(context, sizeof(*context));
 	}
 	MEMSET_BZERO(digest, SHA256_DIGEST_LENGTH);
 	return buffer;
@@ -930,8 +939,17 @@ void SHA512_Last(R_SHA512_CTX* context) {
 		*context->buffer = 0x80;
 	}
 	/* Store the length of input data (in bits): */
+#if WEAK_ALIASING
 	*(sha2_word64*)&context->buffer[SHA512_SHORT_BLOCK_LENGTH] = context->bitcount[1];
 	*(sha2_word64*)&context->buffer[SHA512_SHORT_BLOCK_LENGTH+8] = context->bitcount[0];
+#else
+	{
+		sha2_word64 *p = (sha2_word64*)((ut8*)context->buffer+SHA512_SHORT_BLOCK_LENGTH);
+		*p = (sha2_word64)context->bitcount[1];
+		p = (sha2_word64*)((ut8*)context->buffer+SHA512_SHORT_BLOCK_LENGTH+8);
+		*p = (sha2_word64)context->bitcount[0];
+	}
+#endif
 
 	/* Final transform: */
 	SHA512_Transform(context, (sha2_word64*)context->buffer);
@@ -963,7 +981,7 @@ void SHA512_Final(sha2_byte digest[], R_SHA512_CTX* context) {
 	}
 
 	/* Zero out state data */
-	MEMSET_BZERO(context, sizeof(context));
+	MEMSET_BZERO(context, sizeof(*context));
 }
 
 char *SHA512_End(R_SHA512_CTX* context, char buffer[]) {
@@ -983,7 +1001,7 @@ char *SHA512_End(R_SHA512_CTX* context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-		MEMSET_BZERO(context, sizeof(context));
+		MEMSET_BZERO(context, sizeof(*context));
 	}
 	MEMSET_BZERO(digest, SHA512_DIGEST_LENGTH);
 	return buffer;
@@ -1038,7 +1056,7 @@ void SHA384_Final(sha2_byte digest[], R_SHA384_CTX* context) {
 	}
 
 	/* Zero out state data */
-	MEMSET_BZERO(context, sizeof(context));
+	MEMSET_BZERO(context, sizeof(*context));
 }
 
 char *SHA384_End(R_SHA384_CTX* context, char buffer[]) {
@@ -1058,17 +1076,15 @@ char *SHA384_End(R_SHA384_CTX* context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-		MEMSET_BZERO(context, sizeof(context));
+		MEMSET_BZERO(context, sizeof(*context));
 	}
 	MEMSET_BZERO(digest, SHA384_DIGEST_LENGTH);
 	return buffer;
 }
 
 char* SHA384_Data(const sha2_byte* data, size_t len, char digest[SHA384_DIGEST_STRING_LENGTH]) {
-	R_SHA384_CTX	context;
-
-	SHA384_Init(&context);
-	SHA384_Update(&context, data, len);
-	return SHA384_End(&context, digest);
+	R_SHA384_CTX context;
+	SHA384_Init (&context);
+	SHA384_Update (&context, data, len);
+	return SHA384_End (&context, digest);
 }
-
