@@ -1,12 +1,11 @@
-/* radare - LGPL - Copyright 2009-2013 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2016 - pancake, nibble */
 
-#ifndef _INCLUDE_R_PARSE_H_
-#define _INCLUDE_R_PARSE_H_
+#ifndef R2_PARSE_H
+#define R2_PARSE_H
 
 #include <r_types.h>
-#include <r_flags.h>
+#include <r_flag.h>
 #include <r_anal.h>
-#include <list.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -17,13 +16,19 @@ extern "C" {
 
 R_LIB_VERSION_HEADER(r_parse);
 
+typedef RList* (*RAnalVarList)(RAnal *anal, RAnalFunction *fcn, int kind);
+
 typedef struct r_parse_t {
 	void *user;
 	int flagspace;
 	int notin_flagspace;
+	bool relsub; // replace rip relative expressions in instruction
 	struct r_parse_plugin_t *cur;
 	RAnal *anal; // weak anal ref
+	RAnalHint *hint; // weak anal ref
 	RList *parsers;
+	RAnalVarList varlist;
+	RAnalBind analb;
 } RParse;
 
 typedef struct r_parse_plugin_t {
@@ -33,13 +38,13 @@ typedef struct r_parse_plugin_t {
 	int (*fini)(void *user);
 	int (*parse)(RParse *p, const char *data, char *str);
 	int (*assemble)(RParse *p, char *data, char *str);
-	int (*filter)(RParse *p, RFlag *f, char *data, char *str, int len);
-	int (*varsub)(RParse *p, RAnalFunction *f, char *data, char *str, int len);
-	struct list_head list;
+	int (*filter)(RParse *p, RFlag *f, char *data, char *str, int len, bool big_endian);
+	bool (*varsub)(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len);
+	int (*replace)(int argc, const char *argv[], char *newstr);
 } RParsePlugin;
 
 #ifdef R_API
-R_API struct r_parse_t *r_parse_new();
+R_API struct r_parse_t *r_parse_new(void);
 R_API void r_parse_free(RParse *p);
 R_API void r_parse_set_user_ptr(RParse *p, void *user);
 R_API int r_parse_add(RParse *p, struct r_parse_plugin_t *foo);
@@ -47,8 +52,8 @@ R_API int r_parse_list(RParse *p);
 R_API int r_parse_use(RParse *p, const char *name);
 R_API int r_parse_parse(RParse *p, const char *data, char *str);
 R_API int r_parse_assemble(RParse *p, char *data, char *str);
-R_API int r_parse_filter(RParse *p, RFlag *f, char *data, char *str, int len);
-R_API int r_parse_varsub(RParse *p, RAnalFunction *f, char *data, char *str, int len);
+R_API int r_parse_filter(RParse *p, RFlag *f, char *data, char *str, int len, bool big_endian);
+R_API bool r_parse_varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len);
 R_API char *r_parse_c_string(const char *code);
 R_API char *r_parse_c_file(const char *path);
 R_API int r_parse_is_c_file (const char *file);
@@ -57,9 +62,13 @@ R_API int r_parse_is_c_file (const char *file);
 extern struct r_parse_plugin_t r_parse_plugin_dummy;
 extern struct r_parse_plugin_t r_parse_plugin_att2intel;
 extern struct r_parse_plugin_t r_parse_plugin_x86_pseudo;
+extern struct r_parse_plugin_t r_parse_plugin_arm_pseudo;
 extern struct r_parse_plugin_t r_parse_plugin_mips_pseudo;
 extern struct r_parse_plugin_t r_parse_plugin_dalvik_pseudo;
 extern struct r_parse_plugin_t r_parse_plugin_mreplace;
+extern struct r_parse_plugin_t r_parse_plugin_ppc_pseudo;
+extern struct r_parse_plugin_t r_parse_plugin_6502_pseudo;
+extern struct r_parse_plugin_t r_parse_plugin_m68k_pseudo;
 #endif
 
 #ifdef __cplusplus
