@@ -3,7 +3,7 @@
  * Copyright (c) Ian F. Darwin 1986-1995.
  * Software written by Ian F. Darwin and others;
  * maintained 1995-present by Christos Zoulas and others.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +13,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *  
+ *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -70,9 +70,11 @@ static void cvt_64(union VALUETYPE *, const struct r_magic *);
 int file_softmagic(RMagic *ms, const ut8 *buf, size_t nbytes, int mode) {
 	struct mlist *ml;
 	int rv;
-	for (ml = ms->mlist->next; ml != ms->mlist; ml = ml->next)
-		if ((rv = match(ms, ml->magic, ml->nmagic, buf, nbytes, mode)) != 0)
+	for (ml = ms->mlist->next; ml != ms->mlist; ml = ml->next) {
+		if ((rv = match(ms, ml->magic, ml->nmagic, buf, nbytes, mode)) != 0) {
 			return rv;
+		}
+	}
 	return 0;
 }
 
@@ -111,18 +113,18 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 	int firstline = 1; /* a flag to print X\n  X\n- X */
 	int printed_something = 0;
 
-	if (file_check_mem (ms, cont_level) == -1)
+	if (file_check_mem (ms, cont_level) == -1) {
 		return -1;
-
+	}
 	for (magindex = 0; magindex < nmagic; magindex++) {
 		int flush;
 		struct r_magic *m = &magic[magindex];
 
 		if ((m->flag & BINTEST) != mode) {
 			/* Skip sub-tests */
-			while (magic[magindex + 1].cont_level != 0 &&
-			       ++magindex < nmagic)
+			while (magic[magindex + 1].cont_level != 0 && ++magindex < nmagic - 1) {
 				continue;
+			}
 			continue; /* Skip to next top-level test*/
 		}
 
@@ -132,20 +134,26 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 		/* if main entry matches, print it... */
 		flush = !mget(ms, s, m, nbytes, cont_level);
 		if (flush) {
-			if (m->reln == '!')
+			if (m->reln == '!') {
 				flush = 0;
+			}
 		} else {
 			int ret = magiccheck (ms, m);
-			if (ret == -1) return -1;
-			if (!ret) flush++;
+			if (ret == -1) {
+				return -1;
+			}
+			if (!ret) {
+				flush++;
+			}
 		}
 		if (flush) {
-			/* 
+			/*
 			 * main entry didn't match,
 			 * flush its continuations
 			 */
-			while (magindex < nmagic - 1 && magic[magindex + 1].cont_level)
+			while (magindex < nmagic - 1 && magic[magindex + 1].cont_level) {
 				magindex++;
+			}
 			continue;
 		}
 
@@ -164,15 +172,17 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 			return -1;
 
 		/* and any continuations that match */
-		if (file_check_mem(ms, ++cont_level) == -1)
+		if (file_check_mem(ms, ++cont_level) == -1) {
 			return -1;
+		}
 
-		while (magic[magindex+1].cont_level != 0 && ++magindex < nmagic) {
+		while (++magindex < nmagic - 1 && magic[magindex].cont_level != 0) {
 			m = &magic[magindex];
 			ms->line = m->lineno; /* for messages */
 
-			if (cont_level < m->cont_level)
+			if (cont_level < m->cont_level) {
 				continue;
+			}
 			if (cont_level > m->cont_level) {
 				/*
 				 * We're at the end of the level
@@ -191,7 +201,7 @@ static int match(RMagic *ms, struct r_magic *magic, ut32 nmagic, const ut8 *s, s
 			flush = !mget(ms, s, m, nbytes, cont_level);
 			if (flush && m->reln != '!')
 				continue;
-				
+
 			switch (flush ? 1 : magiccheck(ms, m)) {
 			case -1:
 				return -1;
@@ -258,13 +268,13 @@ static int check_fmt(RMagic *ms, struct r_magic *m) {
 	RRegex rx;
 	int rc;
 
-	if (strchr (R_MAGIC_DESC, '%') == NULL)
+	if (!strchr (R_MAGIC_DESC, '%'))
 		return 0;
 
 	rc = r_regex_comp (&rx, "%[-0-9\\.]*s", R_REGEX_EXTENDED|R_REGEX_NOSUB);
 	if (rc) {
 		char errmsg[512];
-		r_regex_error (rc, &rx, errmsg, sizeof (errmsg));
+		r_regex_error (rc, &rx, errmsg, sizeof (errmsg) - 1);
 		file_magerror (ms, "regex error %d, (%s)", rc, errmsg);
 		return -1;
 	} else {
@@ -280,7 +290,7 @@ char * strdupn(const char *str, size_t n) {
 
 	for (len = 0; len < n && str[len]; len++)
 		continue;
-	if ((copy = malloc (len + 1)) == NULL)
+	if (!(copy = malloc (len + 1)))
 		return NULL;
 	(void)memcpy (copy, str, len);
 	copy[len] = '\0';
@@ -292,7 +302,7 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 	float vf;
 	double vd;
 	ut64 t = 0;
- 	char *buf;
+ 	char *buf = NULL;
 	union VALUETYPE *p = &ms->ms_value;
 
   	switch (m->type) {
@@ -304,12 +314,13 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 		case 1:
 			buf = malloc (2);
 			if (snprintf (buf, 2, "%c", (ut8)v)<0) {
-				return -1;
 				free (buf);
+				return -1;
 			}
-			if (file_printf (ms, R_MAGIC_DESC, buf) == -1)
+			if (file_printf (ms, R_MAGIC_DESC, buf) == -1) {
 				free (buf);
 				return -1;
+			}
 			break;
 		default:
 			if (file_printf(ms, R_MAGIC_DESC, (ut8) v) == -1)
@@ -484,7 +495,7 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 		int rval;
 
 		cp = strdupn((const char *)ms->search.s, ms->search.rm_len);
-		if (cp == NULL) {
+		if (!cp) {
 			file_oomem(ms, ms->search.rm_len);
 			return -1;
 		}
@@ -517,7 +528,8 @@ static st32 mprint(RMagic *ms, struct r_magic *m) {
 		file_magerror(ms, "invalid m->type (%d) in mprint()", m->type);
 		return -1;
 	}
-	return(t);
+	free (buf);
+	return t;
 }
 
 #define DO_CVT(fld, cast) \
@@ -621,7 +633,7 @@ static int mconvert(RMagic *ms, struct r_magic *m) {
 	case FILE_BESTRING16:
 	case FILE_LESTRING16: {
 		size_t len;
-		
+
 		/* Null terminate and eat *trailing* return */
 		p->s[sizeof(p->s) - 1] = '\0';
 		len = strlen(p->s);
@@ -758,7 +770,7 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 			const char *buf;	/* start of search region */
 			size_t lines;
 
-			if (s == NULL) {
+			if (!s) {
 				ms->search.s_len = 0;
 				ms->search.s = NULL;
 				return 0;
@@ -775,7 +787,7 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 			}
 			if (lines)
 				last = (const char *)s + nbytes;
-			
+
 			ms->search.s = buf;
 			ms->search.s_len = last - buf;
 			ms->search.offset = offset;
@@ -788,10 +800,10 @@ static int mcopy(RMagic *ms, union VALUETYPE *p, int type, int indir, const ut8
 			const ut8 *esrc = s + nbytes;
 			char *dst = p->s;
 			char *edst = &p->s[sizeof(p->s) - 1];
-			
+
 			if (type == FILE_BESTRING16)
 				src++;
-			
+
 			/* check for pointer overflow */
 			if (src < s) {
 				file_magerror(ms, "invalid offset %zu in mcopy()",
@@ -1168,7 +1180,7 @@ static ut64 file_strncmp(const char *s1, const char *s2, size_t len, ut32 flags)
 	if (0L == flags) { /* normal string: do it fast */
 		while (len-- > 0)
 			if ((v = *b++ - *a++) != '\0')
-				break; 
+				break;
 	} else { /* combine the others */
 		while (len-- > 0) {
 			if ((flags & STRING_IGNORE_LOWERCASE) &&
@@ -1178,7 +1190,7 @@ static ut64 file_strncmp(const char *s1, const char *s2, size_t len, ut32 flags)
 			} else if ((flags & STRING_IGNORE_UPPERCASE) && isupper(*a)) {
 				if ((v = toupper(*b++) - *a++) != '\0')
 					break;
-			} else if ((flags & STRING_COMPACT_BLANK) && isspace(*a)) { 
+			} else if ((flags & STRING_COMPACT_BLANK) && isspace(*a)) {
 				a++;
 				if (isspace(*b++)) {
 					while (isspace(*b))
@@ -1303,7 +1315,7 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 	case FILE_SEARCH: { /* search ms->search.s for the string m->value.s */
 		size_t slen, idx;
 
-		if (ms->search.s == NULL)
+		if (!ms->search.s)
 			return 0;
 
 		slen = R_MIN (m->vallen, sizeof (m->value.s));
@@ -1311,9 +1323,10 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		v = 0;
 
 		for (idx = 0; m->str_range == 0 || idx < m->str_range; idx++) {
+			if ((int)ms->search.offset < 0)
+				break;
 			if (slen + idx > ms->search.s_len)
 				break;
-
 			v = file_strncmp (m->value.s, ms->search.s + idx, slen, m->str_flags);
 			if (v == 0) {	/* found match */
 				ms->search.offset += idx;
@@ -1327,7 +1340,7 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		RRegex rx;
 		char errmsg[512];
 
-		if (ms->search.s == NULL)
+		if (!ms->search.s)
 			return 0;
 
 		l = 0;
@@ -1335,10 +1348,10 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 		    R_REGEX_EXTENDED|R_REGEX_NEWLINE|
 		    ((m->str_flags & STRING_IGNORE_CASE) ? R_REGEX_ICASE : 0));
 		if (rc) {
-			(void)r_regex_error(rc, &rx, errmsg, sizeof(errmsg));
+			(void)r_regex_error(rc, &rx, errmsg, sizeof(errmsg) - 1);
 			file_magerror(ms, "regex error %d, (%s)",
 			    rc, errmsg);
-			v = (ut64)-1;
+			v = (ut64) - 1;
 		} else {
 			RRegexMatch pmatch[1];
 #ifndef R_REGEX_STARTEND
@@ -1358,18 +1371,16 @@ static int magiccheck(RMagic *ms, struct r_magic *m) {
 			case 0:
 				ms->search.s += (int)pmatch[0].rm_so;
 				ms->search.offset += (size_t)pmatch[0].rm_so;
-				ms->search.rm_len =
-				    (size_t)(pmatch[0].rm_eo - pmatch[0].rm_so);
+				ms->search.rm_len = (size_t)(pmatch[0].rm_eo - pmatch[0].rm_so);
 				v = 0;
 				break;
 			case R_REGEX_NOMATCH:
 				v = 1;
 				break;
 			default:
-				(void)r_regex_error(rc, &rx, errmsg, sizeof(errmsg));
-				file_magerror(ms, "regexec error %d, (%s)",
-				    rc, errmsg);
-				v = (ut64)-1;
+				(void)r_regex_error (rc, &rx, errmsg, sizeof (errmsg) - 1);
+				file_magerror (ms, "regexec error %d, (%s)", rc, errmsg);
+				v = UT64_MAX;
 				break;
 			}
 			r_regex_fini (&rx);
