@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2013 - pancake */
+/* radare - LGPL - Copyright 2013-2016 - pancake */
 
 #include "r_util.h"
 #include "r_types.h"
@@ -7,16 +7,18 @@
 /* parse C code and return it in key-value form */
 
 static void appendstring(const char *msg, char **s) {
-	if (!s)
+	if (!s) {
 		printf ("%s\n", msg);
-	else
-	if (*s) {
-		char *p = malloc (strlen (msg) + strlen (*s)+1);
+	} else if (*s) {
+		char *p = malloc (strlen (msg) + strlen (*s) + 1);
+		if (!p) return;
 		strcpy (p, *s);
 		free (*s);
 		*s = p;
-		strcpy (p+strlen (p), msg);
-	} else *s = strdup (msg);
+		strcpy (p + strlen (p), msg);
+	} else {
+		*s = strdup (msg);
+	}
 }
 
 R_API char *r_parse_c_file(const char *path) {
@@ -41,14 +43,14 @@ R_API char *r_parse_c_string(const char *code) {
 }
 
 R_API int r_parse_is_c_file (const char *file) {
-
 	const char *ext = r_str_lchr (file, '.');
 	if (ext) {
-		ext = ext+1;
+		ext = ext + 1;
 		if (!strcmp (ext, "cparse")
 		||  !strcmp (ext, "c")
-		||  !strcmp (ext, "h"))
-			return R_TRUE;
+		||  !strcmp (ext, "h")) {
+			return true;
+		}
 	}
-	return R_FALSE;
+	return false;
 }
