@@ -1,11 +1,11 @@
-/* ported to C by pancake for r2 in 2012-2013 */
+/* ported to C by pancake for r2 in 2012-2014 */
 // TODO: integrate floating point support
 // TODO: do not use global variables
 /*
    Reference Chapter 6:
    "The C++ Programming Language", Special Edition.
-   Bjarne Stroustrup,Addison-Wesley Pub Co; 3 edition (February 15, 2000) 
-    ISBN: 0201700735 
+   Bjarne Stroustrup,Addison-Wesley Pub Co; 3 edition (February 15, 2000)
+    ISBN: 0201700735
  */
 
 #include <r_types.h>
@@ -17,12 +17,27 @@
 /* accessors */
 static inline RNumCalcValue Nset(ut64 v) { RNumCalcValue n; n.d = (double)v; n.n = v; return n; }
 static inline RNumCalcValue Nsetf(double v) { RNumCalcValue n; n.d = v; n.n = (ut64)v; return n; }
-static inline RNumCalcValue Naddf(RNumCalcValue n, double v) { n.d += v; n.n += (ut64)v; return n; }
+//UNUSED static inline RNumCalcValue Naddf(RNumCalcValue n, double v) { n.d += v; n.n += (ut64)v; return n; }
 static inline RNumCalcValue Naddi(RNumCalcValue n, ut64 v) { n.d += (double)v; n.n += v; return n; }
 static inline RNumCalcValue Nsubi(RNumCalcValue n, ut64 v) { n.d -= (double)v; n.n -= v; return n; }
+static inline RNumCalcValue Nneg(RNumCalcValue n) { n.n = ~n.n; return n; }
+static inline RNumCalcValue Norr(RNumCalcValue n, RNumCalcValue v) { n.d = v.d; n.n |= v.n; return n; }
+static inline RNumCalcValue Nxor(RNumCalcValue n, RNumCalcValue v) { n.d = v.d; n.n ^= v.n; return n; }
+static inline RNumCalcValue Nand(RNumCalcValue n, RNumCalcValue v) { n.d = v.d; n.n &= v.n; return n; }
 static inline RNumCalcValue Nadd(RNumCalcValue n, RNumCalcValue v) { n.d += v.d; n.n += v.n; return n; }
 static inline RNumCalcValue Nsub(RNumCalcValue n, RNumCalcValue v) { n.d -= v.d; n.n -= v.n; return n; }
-static inline RNumCalcValue Nmul(RNumCalcValue n, RNumCalcValue v) { n.d *= v.d; n.n *= v.n; return n; }
+static inline RNumCalcValue Nmul(RNumCalcValue n, RNumCalcValue v) {
+	n.d *= v.d;
+	n.n *= v.n;
+	return n;
+}
+static inline RNumCalcValue Nshl(RNumCalcValue n, RNumCalcValue v) { n.d += v.d; n.n <<= v.n; return n; }
+static inline RNumCalcValue Nshr(RNumCalcValue n, RNumCalcValue v) { n.d += v.d; n.n >>= v.n; return n; }
+static inline RNumCalcValue Nmod(RNumCalcValue n, RNumCalcValue v) {
+	if (v.d) n.d = (n.d - (n.d/v.d)); else n.d = 0;
+	if (v.n) n.n %= v.n; else n.n = 0;
+	return n;
+}
 static inline RNumCalcValue Ndiv(RNumCalcValue n, RNumCalcValue v) {
 	if (v.d) n.d /= v.d; else n.d = 0;
 	if (v.n) n.n /= v.n; else n.n = 0;
@@ -44,11 +59,17 @@ static void error(RNum *num, RNumCalc *nc, const char *s) {
 static RNumCalcValue expr(RNum *num, RNumCalc *nc, int get) {
 	RNumCalcValue left = term (num, nc, get);
 	for (;;) {
-		if (nc->curr_tok == RNCPLUS)
-			left = Nadd (left, term (num, nc, 1));
-		else if (nc->curr_tok == RNCMINUS)
-			left = Nsub (left, term (num, nc, 1));
-		else break;
+		switch (nc->curr_tok) {
+		case RNCSHL: left = Nshl (left, term (num, nc, 1)); break;
+		case RNCSHR: left = Nshr (left, term (num, nc, 1)); break;
+		case RNCPLUS: left = Nadd (left, term (num, nc, 1)); break;
+		case RNCMINUS: left = Nsub (left, term (num, nc, 1)); break;
+		case RNCXOR: left = Nxor (left, term (num, nc, 1)); break;
+		case RNCORR: left = Norr (left, term (num, nc, 1)); break;
+		case RNCAND: left = Nand (left, term (num, nc, 1)); break;
+		default:
+			return left;
+		}
 	}
 	return left;
 }
@@ -59,10 +80,18 @@ static RNumCalcValue term(RNum *num, RNumCalc *nc, int get) {
 		if (nc->curr_tok == RNCMUL) {
 			left = Nmul (left, prim (num, nc, 1));
 		} else
-		if (nc->curr_tok == RNCDIV) {
+		if (nc->curr_tok == RNCMOD) {
 			RNumCalcValue d = prim (num, nc, 1);
 			if (!d.d) {
-				error (num, nc, "divide by 0");
+				//error (num, nc, "divide by 0");
+				return d;
+			}
+			left = Nmod (left, d);
+		} else
+		if (nc->curr_tok == RNCDIV) {
+			RNumCalcValue d = prim (num, nc, 1);
+			if (num != NULL && (!d.d || !d.n)) {
+				num->dbz = 1;
 				return d;
 			}
 			left = Ndiv (left, d);
@@ -84,26 +113,37 @@ static RNumCalcValue prim(RNum *num, RNumCalc *nc, int get) {
 		r_str_chop (nc->string_value);
 		v = Nset (r_num_get (num, nc->string_value));
 		get_token (num, nc);
-		if (nc->curr_tok  == RNCASSIGN) 
+		if (nc->curr_tok  == RNCASSIGN) {
 			v = expr (num, nc, 1);
+		}
 		if (nc->curr_tok == RNCINC) Naddi (v, 1);
 		if (nc->curr_tok == RNCDEC) Nsubi (v, 1);
 		return v;
+	case RNCNEG:
+		v = nc->number_value;
+		get_token (num, nc);
+		return Nneg (nc->number_value); //prim (num, nc, 1), 1);
 	case RNCINC: return Naddi (prim (num, nc, 1), 1);
 	case RNCDEC: return Naddi (prim (num, nc, 1), -1);
+	case RNCORR: return Norr (v, prim (num, nc, 1));
 	case RNCMINUS: return Nsub (v, prim (num, nc, 1));
 	case RNCLEFTP:
 		v = expr (num, nc, 1);
-		if (nc->curr_tok == RNCRIGHTP)
+		if (nc->curr_tok == RNCRIGHTP) {
 			get_token (num, nc);
-		else error (num, nc, " ')' expected");
+		} else error (num, nc, " ')' expected");
 	case RNCEND:
+	case RNCXOR:
+	case RNCAND:
 	case RNCPLUS:
+	case RNCMOD:
 	case RNCMUL:
 	case RNCDIV:
 	case RNCPRINT:
 	case RNCASSIGN:
 	case RNCRIGHTP:
+	case RNCSHL:
+	case RNCSHR:
 		return v;
 	//default: error (num, nc, "primary expected");
 	}
@@ -115,14 +155,12 @@ static void cin_putback (RNum *num, RNumCalc *nc, char c) {
 }
 
 R_API const char *r_num_calc_index (RNum *num, const char *p) {
-	if (num == NULL)
-		return NULL;
+	if (!num) return NULL;
 	if (p) {
 		num->nc.calc_buf = p;
 		num->nc.calc_len = strlen (p);
 		num->nc.calc_i = 0;
 	}
-	//if (num->nc.calc_i>num->nc.calc_len) return NULL;
 	return num->nc.calc_buf + num->nc.calc_i;
 }
 
@@ -143,17 +181,18 @@ static int cin_get(RNum *num, RNumCalc *nc, char *c) {
 
 static int cin_get_num(RNum *num, RNumCalc *nc, RNumCalcValue *n) {
 	double d;
-	char str[R_NUMCALC_STRSZ];
+	char str[R_NUMCALC_STRSZ]; // TODO: move into the heap?
 	int i = 0;
 	char c;
 	str[0] = 0;
 	while (cin_get (num, nc, &c)) {
-		if (c!=':' && c!='.' && !isalnum (c)) {
+		if (c!=':' && c!='.' && !isalnum ((ut8)c)) {
 			cin_putback (num, nc, c);
 			break;
 		}
-		if (i<R_NUMCALC_STRSZ)
+		if (i < R_NUMCALC_STRSZ) {
 			str[i++] = c;
+		}
 	}
 	str[i] = 0;
 	*n = Nset (r_num_get (num, str));
@@ -188,7 +227,7 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 	char ch = 0, c = 0;
 
 	do { if (!cin_get (num, nc, &ch)) return nc->curr_tok = RNCEND;
-	} while (ch!='\n' && isspace (ch));
+	} while (ch!='\n' && isspace ((unsigned char)ch));
 
 	switch (ch) {
 	case 0:
@@ -200,18 +239,31 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 			return nc->curr_tok = RNCINC;
 		cin_putback (num, nc, c);
 		return nc->curr_tok = (RNumCalcToken) ch;
+	// negate hack
+	case '~':
+		if (cin_get (num, nc, &c) && c == '-')
+			return nc->curr_tok = RNCNEG;
+		cin_putback (num, nc, c);
+		return nc->curr_tok = (RNumCalcToken) ch;
+	// negative number
 	case '-':
 		if (cin_get (num, nc, &c) && c == '-')
 			return nc->curr_tok = RNCDEC;
 		cin_putback (num, nc, c);
 		return nc->curr_tok = (RNumCalcToken) ch;
+	case '^':
+	case '&':
+	case '|':
 	case '*':
+	case '%':
 	case '/':
 	case '(':
 	case ')':
+	case '<':
+	case '>':
 	case '=':
 		return nc->curr_tok = (RNumCalcToken) ch;
-	case '0':case '1': case '2': case '3': case '4':
+	case '0': case '1': case '2': case '3': case '4':
 	case '5': case '6': case '7': case '8': case '9':
 	case '.':
 		cin_putback (num, nc, ch);
@@ -220,39 +272,44 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 			return 1;
 		}
 		return nc->curr_tok = RNCNUMBER;
-	default:
+
 #define isvalidchar(x) \
 	(isalnum(x) || x==':' || x=='$' || x=='.' || x=='_' || x=='?' || x=='\\' \
-	|| x==' ' || x=='[' || x==']' || x=='}' || x=='{' || x=='/' || (x>='0'&&x<='9'))
-{
+	|| x==' ' || x=='[' || x==']' || x=='}' || x=='{' || (x>='0'&&x<='9'))
+
+	default:
+		{
 			int i = 0;
-			nc->string_value[i++] = ch;
+#define stringValueAppend(x) { \
+	const size_t max = sizeof (nc->string_value) - 1; \
+	if (i < max) nc->string_value[i++] = x; \
+	else nc->string_value[max] = 0; \
+}
+			stringValueAppend(ch);
 			if (ch == '[') {
 				while (cin_get (num, nc, &ch) && ch!=']') {
-					if (i>=R_NUMCALC_STRSZ) {
+					if (i > R_NUMCALC_STRSZ - 1) {
 						error (num, nc, "string too long");
 						return 0;
 					}
-					nc->string_value[i++] = ch;
+					stringValueAppend(ch);
 				}
-				nc->string_value[i++] = ch;
+				stringValueAppend(ch);
 			} else {
-				while (cin_get (num, nc, &ch) && isvalidchar (ch)) {
+				while (cin_get (num, nc, &ch) && isvalidchar ((unsigned char)ch)) {
 					if (i>=R_NUMCALC_STRSZ) {
 						error (num, nc, "string too long");
 						return 0;
 					}
-					nc->string_value[i++] = ch;
+					stringValueAppend(ch);
 				}
 			}
-			nc->string_value[i] = 0;
-			cin_putback (num, nc, ch);
-
+			stringValueAppend(0);
+			if (ch!='\'') {
+				cin_putback (num, nc, ch);
+			}
 			return nc->curr_tok = RNCNAME;
-}
-		//}
-		error (num, nc, "bad token");
-		return nc->curr_tok = RNCPRINT;
+		}
 	}
 }
 
@@ -263,15 +320,17 @@ static void load_token(RNum *num, RNumCalc *nc, const char *s) {
 	nc->calc_err = NULL;
 }
 
-R_API ut64 r_num_calc (RNum *num, const char *str, const char **err) {
+R_API ut64 r_num_calc(RNum *num, const char *str, const char **err) {
 	RNumCalcValue n;
 	RNumCalc *nc, nc_local;
 	if (!str || !*str)
 		return 0LL;
-
-	if (num == NULL)
+	if (num) {
+		nc = &num->nc;
+		num->dbz = 0;
+	} else {
 		nc = &nc_local;
-	else nc = &num->nc;
+	}
 
 	/* init */
 	nc->curr_tok = RNCPRINT;
@@ -291,9 +350,14 @@ R_API ut64 r_num_calc (RNum *num, const char *str, const char **err) {
 	//if (nc->curr_tok == RNCEND) return 0LL; // XXX: Error
 	//if (nc->curr_tok == RNCPRINT) //return 0LL; // XXX: the fuck
 	//	n = expr (num, nc, 0);
+#if 0
 	if (n.d != ((double)(ut64)n.d)) {
 		if (num) num->fvalue = n.d;
 	} else if (num) num->fvalue = (double)n.n;
+#endif
+	if (num) {
+		num->fvalue = n.d;
+	}
 	return n.n;
 }
 
