@@ -169,6 +169,9 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 	m->offp = string;
 	m->beginp = start;
 	m->endp = stop;
+
+	if (m->g->nstates * 4 < m->g->nstates)
+		return R_REGEX_NOMATCH;
 	STATESETUP(m, 4);
 	SETUP(m->st);
 	SETUP(m->fresh);
@@ -179,7 +182,7 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 	/* this loop does only one repetition except for backrefs */
 	for (;;) {
 		endp = fast(m, start, stop, gf, gl);
-		if (endp == NULL) {		/* a miss */
+		if (!endp) {		/* a miss */
 			free(m->pmatch);
 			free(m->lastpos);
 			STATETEARDOWN(m);
@@ -202,10 +205,14 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 			break;		/* no further info needed */
 
 		/* oh my, he wants the subexpressions... */
-		if (m->pmatch == NULL)
+		if (!m->pmatch) {
+			if ((m->g->nsub + 1) * sizeof(RRegexMatch) < m->g->nsub) {
+				return R_REGEX_ESPACE;
+			}
 			m->pmatch = (RRegexMatch *)malloc((m->g->nsub + 1) *
 							sizeof(RRegexMatch));
-		if (m->pmatch == NULL) {
+		}
+		if (!m->pmatch) {
 			STATETEARDOWN(m);
 			return(R_REGEX_ESPACE);
 		}
@@ -215,10 +222,16 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 			NOTE("dissecting");
 			dp = dissect(m, m->coldp, endp, gf, gl);
 		} else {
-			if (g->nplus > 0 && m->lastpos == NULL)
+			if (g->nplus > 0 && !m->lastpos) {
+				if ((g->nplus + 1) * sizeof(char *) < g->nplus) {
+					free (m->pmatch);
+					STATETEARDOWN(m);
+					return R_REGEX_ESPACE;
+				}
 				m->lastpos = (char **)malloc((g->nplus+1) *
 							sizeof(char *));
-			if (g->nplus > 0 && m->lastpos == NULL) {
+			}
+			if (g->nplus > 0 && !m->lastpos) {
 				free(m->pmatch);
 				STATETEARDOWN(m);
 				return(R_REGEX_ESPACE);
@@ -237,7 +250,7 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 				break;		/* defeat */
 			NOTE("backoff");
 			endp = slow(m, m->coldp, endp-1, gf, gl);
-			if (endp == NULL)
+			if (!endp)
 				break;		/* defeat */
 			/* try it on a shorter possibility */
 #ifndef NDEBUG
@@ -249,7 +262,7 @@ matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch pmatch[],
 			NOTE("backoff dissect");
 			dp = backref(m, m->coldp, endp, gf, gl, (sopno)0, 0);
 		}
-		assert(dp == NULL || dp == endp);
+		assert(!dp || dp == endp);
 		if (dp != NULL)		/* found a shorter one */
 			break;
 
@@ -386,12 +399,12 @@ dissect(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 			oldssp = ssp;
 			for (;;) {	/* find last match of innards */
 				sep = slow(m, ssp, rest, ssub, esub);
-				if (sep == NULL || sep == ssp)
+				if (!sep || sep == ssp)
 					break;	/* failed or matched null */
 				oldssp = ssp;	/* on to next try */
 				ssp = sep;
 			}
-			if (sep == NULL) {
+			if (!sep) {
 				/* last successful match */
 				sep = ssp;
 				ssp = oldssp;
@@ -481,7 +494,7 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 	size_t len;
 	int hard;
 	sop s;
-	regoff_t offsave;
+	ut64 offsave;
 	cset *cs;
 
 	AT("back", start, stop, startst, stopst);
@@ -526,8 +539,8 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 					(sp < m->endp && *(sp-1) == '\n' &&
 						(m->g->cflags&R_REGEX_NEWLINE)) ||
 					(sp > m->beginp &&
-							!ISWORD(*(sp-1))) ) &&
-					(sp < m->endp && ISWORD(*sp)) )
+							!ISWORD((unsigned char)*(sp-1))) ) &&
+					(sp < m->endp && ISWORD((unsigned char)*sp)) )
 				{ /* yes */ }
 			else
 				return(NULL);
@@ -536,8 +549,8 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 			if (( (sp == m->endp && !(m->eflags&R_REGEX_NOTEOL)) ||
 					(sp < m->endp && *sp == '\n' &&
 						(m->g->cflags&R_REGEX_NEWLINE)) ||
-					(sp < m->endp && !ISWORD(*sp)) ) &&
-					(sp > m->beginp && ISWORD(*(sp-1))) )
+					(sp < m->endp && !ISWORD((unsigned char)*sp)) ) &&
+					(sp > m->beginp && ISWORD((unsigned char)*(sp-1))) )
 				{ /* yes */ }
 			else
 				return(NULL);
@@ -605,7 +618,7 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 		/* try another pass */
 		m->lastpos[lev] = sp;
 		dp = backref(m, sp, stop, ss-OPND(s)+1, stopst, lev, rec);
-		if (dp == NULL)
+		if (!dp)
 			return(backref(m, sp, stop, ss+1, stopst, lev-1, rec));
 		else
 			return(dp);
@@ -690,7 +703,7 @@ fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 		/* next character */
 		lastc = c;
 		c = (p == m->endp) ? OUT : *p;
-		if (EQ(st, fresh))
+		if (st==fresh)
 			coldp = p;
 
 		/* is there an EOL and/or BOL between lastc and c? */
@@ -744,8 +757,7 @@ fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 	m->coldp = coldp;
 	if (ISSET(st, stopst))
 		return(p+1);
-	else
-		return(NULL);
+	return NULL;
 }
 
 /*
@@ -969,7 +981,7 @@ print(struct match *m, char *caption, states st, int ch, FILE *d)
 	(void)fprintf(d, "\n");
 }
 
-/* 
+/*
  - at - print current situation
  */
 static void
