@@ -140,9 +140,12 @@ static char nuls[10];		/* place to point scanner in event of error */
 R_API int r_regex_match (const char *pattern, const char *flags, const char *text) {
 	int ret;
 	RRegex rx;
-	if (r_regex_comp (&rx, pattern, r_regex_flags (flags)))
-		return -1;
-	ret = r_regex_exec (&rx, text, 0, 0, 0);
+	int re_flags = r_regex_flags (flags);
+	if (r_regex_comp (&rx, pattern, re_flags)) {
+		eprintf ("FAIL TO COMPILE %s\n", pattern);
+		return 0;
+	}
+	ret = r_regex_exec (&rx, text, 0, 0, re_flags);
 	r_regex_fini (&rx);
 	return ret? 0: 1;
 #if 0
@@ -156,9 +159,11 @@ R_API int r_regex_match (const char *pattern, const char *flags, const char *tex
 
 R_API RRegex *r_regex_new (const char *pattern, const char *flags) {
 	RRegex rx, *r;
+	memset(&rx, 0, sizeof(RRegex));
 	if (r_regex_comp (&rx, pattern, r_regex_flags (flags)))
 		return NULL;
 	r = malloc (sizeof (RRegex));
+	if (!r) return NULL;
 	memcpy (r, &rx, sizeof (RRegex));
 	return r;
 }
@@ -183,20 +188,16 @@ R_API void r_regex_fini(RRegex *preg) {
 		return;			/* nice to complain, but hard */
 
 	g = preg->re_g;
-	if (g == NULL || g->magic != MAGIC2)	/* oops again */
+	if (!g || g->magic != MAGIC2)	/* oops again */
 		return;
 	preg->re_magic = 0;		/* mark it invalid */
 	g->magic = 0;			/* mark it invalid */
 
-	if (g->strip != NULL)
-		free((char *)g->strip);
-	if (g->sets != NULL)
-		free((char *)g->sets);
-	if (g->setbits != NULL)
-		free((char *)g->setbits);
-	if (g->must != NULL)
-		free(g->must);
-	free((char *)g);
+	free(g->strip);
+	free(g->sets);
+	free(g->setbits);
+	free(g->must);
+	free(g);
 }
 
 R_API void r_regex_free(RRegex *preg) {
@@ -206,10 +207,9 @@ R_API void r_regex_free(RRegex *preg) {
 
 /*
  - regcomp - interface for parser and compilation
+ - 0 success, otherwise R_REGEX_something
  */
-R_API int			/* 0 success, otherwise R_REGEX_something */
-r_regex_comp(RRegex *preg, const char *pattern, int cflags)
-{
+R_API int r_regex_comp(RRegex *preg, const char *pattern, int cflags) {
 	struct parse pa;
 	struct re_guts *g;
 	struct parse *p = &pa;
@@ -221,28 +221,32 @@ r_regex_comp(RRegex *preg, const char *pattern, int cflags)
 #	define	GOODFLAGS(f)	((f)&~R_REGEX_DUMP)
 #endif
 
-	cflags = GOODFLAGS(cflags);
-	if ((cflags&R_REGEX_EXTENDED) && (cflags&R_REGEX_NOSPEC))
-		return(R_REGEX_INVARG);
+	cflags = GOODFLAGS (cflags);
+	if ((cflags & R_REGEX_EXTENDED) && (cflags & R_REGEX_NOSPEC))
+		return R_REGEX_INVARG;
 
-	if (cflags&R_REGEX_PEND) {
+	if (cflags & R_REGEX_PEND) {
 		if (preg->re_endp < pattern)
 			return(R_REGEX_INVARG);
 		len = preg->re_endp - pattern;
-	} else
-		len = strlen((char *)pattern);
+	} else len = strlen ((char *)pattern);
 
 	/* do the mallocs early so failure handling is easy */
-	g = (struct re_guts *)malloc(sizeof(struct re_guts) +
-							(NC-1)*sizeof(cat_t));
-	if (g == NULL)
-		return(R_REGEX_ESPACE);
-	p->ssize = len/(size_t)2*(size_t)3 + (size_t)1;	/* ugh */
-	p->strip = (sop *)calloc(p->ssize, sizeof(sop));
+	g = (struct re_guts *)calloc (sizeof (struct re_guts) + (NC - 1), sizeof (cat_t));
+	if (!g) {
+		return R_REGEX_ESPACE;
+	}
+	preg->re_flags = cflags;
+	p->ssize = len / (size_t)2 * (size_t)3 + (size_t)1;	/* ugh */
+	p->strip = (sop *)calloc (p->ssize, sizeof(sop));
+	if (!p->strip) {
+		free ((char *)g);
+		return R_REGEX_ESPACE;
+	}
 	p->slen = 0;
-	if (p->strip == NULL) {
-		free((char *)g);
-		return(R_REGEX_ESPACE);
+	if (!p->strip) {
+		free ((char *)g);
+		return R_REGEX_ESPACE;
 	}
 
 	/* set things up */
@@ -267,19 +271,18 @@ r_regex_comp(RRegex *preg, const char *pattern, int cflags)
 	g->mlen = 0;
 	g->nsub = 0;
 	g->ncategories = 1;	/* category 0 is "everything else" */
-	g->categories = &g->catspace[-(CHAR_MIN)];
+	g->categories = &g->catspace[-(CHAR_MIN)]; // WTF
 	(void) memset((char *)g->catspace, 0, NC*sizeof(cat_t));
 	g->backrefs = 0;
 
 	/* do it */
 	EMIT(OEND, 0);
 	g->firststate = THERE();
-	if (cflags&R_REGEX_EXTENDED)
+	if (cflags & R_REGEX_EXTENDED)
 		p_ere(p, OUT);
-	else if (cflags&R_REGEX_NOSPEC)
-		p_str(p);
-	else
-		p_bre(p, OUT, OUT);
+	else if (cflags & R_REGEX_NOSPEC)
+		p_str (p);
+	else p_bre (p, OUT, OUT);
 	EMIT(OEND, 0);
 	g->laststate = THERE();
 
@@ -299,17 +302,15 @@ r_regex_comp(RRegex *preg, const char *pattern, int cflags)
 #endif
 
 	/* win or lose, we're done */
-	if (p->error != 0)	/* lose */
-		r_regex_fini(preg);
-	return(p->error);
+	if (p->error) /* lose */
+		r_regex_fini (preg);
+	return p->error;
 }
 
 /*
  - p_ere - ERE parser top level, concatenation and alternation
  */
-static void
-p_ere(struct parse *p, int stop)	/* character this ERE should end at */
-{
+static void p_ere(struct parse *p, int stop) { /* character this ERE should end at */
 	char c;
 	sopno prevback = 0;
 	sopno prevfwd = 0;
@@ -350,9 +351,7 @@ p_ere(struct parse *p, int stop)	/* character this ERE should end at */
 /*
  - p_ere_exp - parse one subERE, an atom possibly followed by a repetition op
  */
-static void
-p_ere_exp(struct parse *p)
-{
+static void p_ere_exp(struct parse *p) {
 	char c;
 	sopno pos;
 	int count;
@@ -460,7 +459,7 @@ p_ere_exp(struct parse *p)
 				count2 = p_count(p);
 				REQUIRE(count <= count2, R_REGEX_BADBR);
 			} else		/* single number with comma */
-				count2 = INFINITY;
+				count2 = INTFINITY;
 		} else		/* just a single number */
 			count2 = count;
 		repeat(p, pos, count, count2);
@@ -485,9 +484,7 @@ p_ere_exp(struct parse *p)
 /*
  - p_str - string (no metacharacters) "parser"
  */
-static void
-p_str(struct parse *p)
-{
+static void p_str(struct parse *p) {
 	REQUIRE(MORE(), R_REGEX_EMPTY);
 	while (MORE())
 		ordinary(p, GETNEXT());
@@ -503,8 +500,7 @@ p_str(struct parse *p)
  * category in such cases.  This is fairly harmless; not worth fixing.
  * The amount of lookahead needed to avoid this kludge is excessive.
  */
-static void
-p_bre(struct parse *p,
+static void p_bre(struct parse *p,
     int end1,		/* first terminating character */
     int end2)		/* second terminating character */
 {
@@ -631,7 +627,7 @@ p_simp_re(struct parse *p,
 				count2 = p_count(p);
 				REQUIRE(count <= count2, R_REGEX_BADBR);
 			} else		/* single number with comma */
-				count2 = INFINITY;
+				count2 = INTFINITY;
 		} else		/* just a single number */
 			count2 = count;
 		repeat(p, pos, count, count2);
@@ -671,9 +667,7 @@ p_count(struct parse *p)
  * Note a significant property of this code:  if the allocset() did SETERROR,
  * no set operations are done.
  */
-static void
-p_bracket(struct parse *p)
-{
+static void p_bracket(struct parse *p) {
 	cset *cs;
 	int invert = 0;
 
@@ -689,7 +683,7 @@ p_bracket(struct parse *p)
 		return;
 	}
 
-	if ((cs = allocset(p)) == NULL) {
+	if (!(cs = allocset(p))) {
 		/* allocset did set error status in p */
 		return;
 	}
@@ -738,21 +732,20 @@ p_bracket(struct parse *p)
 			mcinvert(p, cs);
 	}
 
-	assert(cs->multis == NULL);		/* xxx */
+	assert(!cs->multis);		/* xxx */
 
 	if (nch(p, cs) == 1) {		/* optimize singleton sets */
 		ordinary(p, firstch(p, cs));
 		freeset(p, cs);
-	} else
+	} else {
 		EMIT(OANYOF, freezeset(p, cs));
+	}
 }
 
 /*
  - p_b_term - parse one term of a bracketed character list
  */
-static void
-p_b_term(struct parse *p, cset *cs)
-{
+static void p_b_term(struct parse *p, cset *cs) {
 	char c;
 	char start = 0, finish;
 	int i;
@@ -813,22 +806,20 @@ p_b_term(struct parse *p, cset *cs)
 /*
  - p_b_cclass - parse a character-class name and deal with it
  */
-static void
-p_b_cclass(struct parse *p, cset *cs)
-{
+static void p_b_cclass(struct parse *p, cset *cs) {
 	char *sp = p->next;
 	struct cclass *cp;
 	size_t len;
 	char *u;
 	char c;
 
-	while (MORE() && isalpha(PEEK()))
+	while (MORE() && isalpha((unsigned char)PEEK()))
 		NEXT();
 	len = p->next - sp;
 	for (cp = cclasses; cp->name != NULL; cp++)
 		if (strncmp(cp->name, sp, len) == 0 && cp->name[len] == '\0')
 			break;
-	if (cp->name == NULL) {
+	if (!cp->name) {
 		/* oops, didn't find it */
 		SETERROR(R_REGEX_ECTYPE);
 		return;
@@ -846,9 +837,7 @@ p_b_cclass(struct parse *p, cset *cs)
  *
  * This implementation is incomplete. xxx
  */
-static void
-p_b_eclass(struct parse *p, cset *cs)
-{
+static void p_b_eclass(struct parse *p, cset *cs) {
 	char c;
 
 	c = p_b_coll_elem(p, '=');
@@ -921,9 +910,7 @@ othercase(int ch)
  *
  * Boy, is this implementation ever a kludge...
  */
-static void
-bothcases(struct parse *p, int ch)
-{
+static void bothcases(struct parse *p, int ch) {
 	char *oldnext = p->next;
 	char *oldend = p->end;
 	char bracket[3];
@@ -989,13 +976,13 @@ static void
 repeat(struct parse *p,
     sopno start,		/* operand from here to end of strip */
     int from,			/* repeated from this number */
-    int to)			/* to this number of times (maybe INFINITY) */
+    int to)			/* to this number of times (maybe INTFINITY) */
 {
 	sopno finish = HERE();
 #	define	N	2
 #	define	INF	3
 #	define	REP(f, t)	((f)*8 + (t))
-#	define	MAP(n)	(((n) <= 1) ? (n) : ((n) == INFINITY) ? INF : N)
+#	define	MAP(n)	(((n) <= 1) ? (n) : ((n) == INTFINITY) ? INF : N)
 	sopno copy;
 
 	if (p->error != 0)	/* head off possible runaway recursion */
@@ -1068,9 +1055,7 @@ seterr(struct parse *p, int e)
 /*
  - allocset - allocate a set of characters for []
  */
-static cset *
-allocset(struct parse *p)
-{
+static cset * allocset(struct parse *p) {
 	int no = p->g->ncsets++;
 	size_t nc;
 	size_t nbytes;
@@ -1087,12 +1072,12 @@ allocset(struct parse *p)
 		nbytes = nc / CHAR_BIT * css;
 
 		ptr = (cset *)realloc((char *)p->g->sets, nc * sizeof(cset));
-		if (ptr == NULL)
+		if (!ptr)
 			goto nomem;
 		p->g->sets = ptr;
 
 		ptr = (ut8 *)realloc((char *)p->g->setbits, nbytes);
-		if (ptr == NULL)
+		if (!ptr)
 			goto nomem;
 		p->g->setbits = ptr;
 
@@ -1102,7 +1087,7 @@ allocset(struct parse *p)
 		(void) memset((char *)p->g->setbits + (nbytes - css), 0, css);
 	}
 	/* XXX should not happen */
-	if (p->g->sets == NULL || p->g->setbits == NULL)
+	if (!p->g->sets || !p->g->setbits)
 		goto nomem;
 
 	cs = &p->g->sets[no];
@@ -1127,9 +1112,7 @@ nomem:
 /*
  - freeset - free a now-unused set
  */
-static void
-freeset(struct parse *p, cset *cs)
-{
+static void freeset(struct parse *p, cset *cs) {
 	int i;
 	cset *top = &p->g->sets[p->g->ncsets];
 	size_t css = (size_t)p->g->csetsize;
@@ -1196,9 +1179,7 @@ firstch(struct parse *p, cset *cs)
 /*
  - nch - number of characters in a set
  */
-static int
-nch(struct parse *p, cset *cs)
-{
+static int nch(struct parse *p, cset *cs) {
 	int i;
 	size_t css = (size_t)p->g->csetsize;
 	int n = 0;
@@ -1212,15 +1193,13 @@ nch(struct parse *p, cset *cs)
 /*
  - mcadd - add a collating element to a cset
  */
-static void
-mcadd( struct parse *p, cset *cs, char *cp)
-{
+static void mcadd( struct parse *p, cset *cs, char *cp) {
 	size_t oldend = cs->smultis;
 	void *np;
 
 	cs->smultis += strlen(cp) + 1;
 	np = realloc(cs->multis, cs->smultis);
-	if (np == NULL) {
+	if (!np) {
 		if (cs->multis)
 			free(cs->multis);
 		cs->multis = NULL;
@@ -1229,7 +1208,7 @@ mcadd( struct parse *p, cset *cs, char *cp)
 	}
 	cs->multis = np;
 
-	strlcpy(cs->multis + oldend - 1, cp, cs->smultis - oldend + 1);
+	STRLCPY(cs->multis + oldend - 1, cp, cs->smultis - oldend + 1);
 }
 
 /*
@@ -1239,10 +1218,8 @@ mcadd( struct parse *p, cset *cs, char *cp)
  * is deferred.
  */
 /* ARGSUSED */
-static void
-mcinvert(struct parse *p, cset *cs)
-{
-	assert(cs->multis == NULL);	/* xxx */
+static void mcinvert(struct parse *p, cset *cs) {
+	assert(!cs->multis);	/* xxx */
 }
 
 /*
@@ -1252,10 +1229,8 @@ mcinvert(struct parse *p, cset *cs)
  * is deferred.
  */
 /* ARGSUSED */
-static void
-mccase(struct parse *p, cset *cs)
-{
-	assert(cs->multis == NULL);	/* xxx */
+static void mccase(struct parse *p, cset *cs) {
+	assert(!cs->multis);	/* xxx */
 }
 
 /*
@@ -1299,17 +1274,17 @@ samesets(struct re_guts *g, int c1, int c2)
 static void
 categorize(struct parse *p, struct re_guts *g)
 {
-	cat_t *cats = g->categories;
-	int c;
-	int c2;
+	cat_t *cats = g? g->categories : NULL;
+	unsigned int c;
+	unsigned int c2;
 	cat_t cat;
 
 	/* avoid making error situations worse */
-	if (p->error != 0)
+	if (!p || p->error != 0 || !cats )
 		return;
 
 	for (c = CHAR_MIN; c <= CHAR_MAX; c++)
-		if (cats[c] == 0 && isinsets(g, c)) {
+		if ( *(cats+c) && isinsets(g, c)) {
 			cat = g->ncategories++;
 			cats[c] = cat;
 			for (c2 = c+1; c2 <= CHAR_MAX; c2++)
@@ -1427,7 +1402,7 @@ enlarge(struct parse *p, sopno size)
 		return;
 
 	sp = (sop *)realloc(p->strip, size*sizeof(sop));
-	if (sp == NULL) {
+	if (!sp) {
 		SETERROR(R_REGEX_ESPACE);
 		return;
 	}
@@ -1443,7 +1418,7 @@ stripsnug(struct parse *p, struct re_guts *g)
 {
 	g->nstates = p->slen;
 	g->strip = (sop *)realloc((char *)p->strip, p->slen * sizeof(sop));
-	if (g->strip == NULL) {
+	if (!g->strip) {
 		SETERROR(R_REGEX_ESPACE);
 		g->strip = p->strip;
 	}
@@ -1517,7 +1492,7 @@ findmust(struct parse *p, struct re_guts *g)
 
 	/* turn it into a character string */
 	g->must = malloc((size_t)g->mlen + 1);
-	if (g->must == NULL) {		/* argh; just forget it */
+	if (!g->must) {		/* argh; just forget it */
 		g->mlen = 0;
 		return;
 	}
