@@ -11,10 +11,10 @@ void buffer_init(buffer *s, BufferOp op, int fd, char *buf, ut32 len) {
 }
 
 static int allwrite(BufferOp op, int fd, const char *buf, ut32 len) {
-	int w;
+	ut32 w;
 	while (len > 0) {
 		w = op (fd, buf, len);
-		if (w < 0)
+		if (w != len)
 			return 0;
 		buf += w;
 		len -= w;
@@ -31,13 +31,15 @@ int buffer_flush(buffer *s) {
 
 int buffer_putalign(buffer *s, const char *buf, ut32 len) {
 	ut32 n;
+	if (!s || !s->x || !buf)
+		return 0;
 	while (len > (n = s->n - s->p)) {
-		byte_copy (s->x + s->p, n, buf);
+		memcpy (s->x + s->p, buf, n);
 		s->p += n; buf += n; len -= n;
 		if (!buffer_flush (s)) return 0;
 	}
 	/* now len <= s->n - s->p */
-	byte_copy (s->x + s->p, len, buf);
+	memcpy (s->x + s->p, buf, len);
 	s->p += len;
 	return 1;
 }
