@@ -135,7 +135,7 @@ PUB_FUNC char *pstrncpy(char *out, const char *in, size_t num)
 PUB_FUNC char *tcc_basename(const char *name)
 {
     char *p = strchr(name, 0);
-    while (p > name && !IS_DIRSEP(p[-1]))
+    while (p && p > name && !IS_DIRSEP(p[-1]))
         --p;
     return p;
 }
@@ -388,7 +388,6 @@ ST_FUNC int tcc_open(TCCState *s1, const char *filename)
 static int tcc_compile(TCCState *s1)
 {
     Sym *define_start;
-    SValue *pvtop;
 
 #ifdef INC_DEBUG
     printf("%s: **** new file\n", file->filename);
@@ -441,14 +440,16 @@ static int tcc_compile(TCCState *s1)
         ch = file->buf_ptr[0];
         tok_flags = TOK_FLAG_BOL | TOK_FLAG_BOF;
         parse_flags = PARSE_FLAG_PREPROCESS | PARSE_FLAG_TOK_NUM;
-        pvtop = vtop;
+        //pvtop = vtop;
         next();
         decl(VT_CONST);
         if (tok != TOK_EOF)
             expect("declaration");
+#if 0
         if (pvtop != vtop)
-            tcc_warning("internal compiler error: vstack leak? (%d)", vtop - pvtop);
-
+            fprintf (stderr, "internal compiler error:"
+		" vstack leak? (%d)", vtop - pvtop);
+#endif
     }
 
     s1->error_set_jmp_enabled = 0;
@@ -813,6 +814,7 @@ typedef struct FlagDef {
     const char *name;
 } FlagDef;
 
+#if 0
 static const FlagDef warning_defs[] = {
     { offsetof(TCCState, warn_unsupported), 0, "unsupported" },
     { offsetof(TCCState, warn_write_strings), 0, "write-strings" },
@@ -820,6 +822,7 @@ static const FlagDef warning_defs[] = {
     { offsetof(TCCState, warn_implicit_function_declaration), WD_ALL,
       "implicit-function-declaration" },
 };
+#endif
 
 ST_FUNC int set_flag(TCCState *s, const FlagDef *flags, int nb_flags,
                     const char *name, int value)
@@ -845,53 +848,16 @@ ST_FUNC int set_flag(TCCState *s, const FlagDef *flags, int nb_flags,
     return 0;
 }
 
-/* set/reset a warning */
-static int tcc_set_warning(TCCState *s, const char *warning_name, int value)
-{
-    int i;
-    const FlagDef *p;
-
-    if (!strcmp(warning_name, "all")) {
-        for(i = 0, p = warning_defs; i < countof(warning_defs); i++, p++) {
-            if (p->flags & WD_ALL)
-                *(int *)((uint8_t *)s + p->offset) = 1;
-        }
-        return 0;
-    } else {
-        return set_flag(s, warning_defs, countof(warning_defs),
-                        warning_name, value);
-    }
-}
-
+#if 0
 static const FlagDef flag_defs[] = {
     { offsetof(TCCState, char_is_unsigned), 0, "unsigned-char" },
     { offsetof(TCCState, char_is_unsigned), FD_INVERT, "signed-char" },
     { offsetof(TCCState, nocommon), FD_INVERT, "common" },
     { offsetof(TCCState, leading_underscore), 0, "leading-underscore" },
 };
+#endif
+void (*tcc_cb)(const char *, char **);
 
-/* set/reset a flag */
-static int tcc_set_flag(TCCState *s, const char *flag_name, int value)
-{
-    return set_flag(s, flag_defs, countof(flag_defs),
-                    flag_name, value);
-}
-
-
-static int strstart(const char *val, const char **str)
-{
-    const char *p, *q;
-    p = *str;
-    q = val;
-    while (*q) {
-        if (*p != *q)
-            return 0;
-        p++;
-        q++;
-    }
-    *str = p;
-    return 1;
-}
 PUB_FUNC void tcc_set_callback (TCCState *s, void (*cb)(const char *,char**), char **p) {
 	tcc_cb = cb;
 	tcc_cb_ptr = p;
