@@ -21,6 +21,7 @@
 #ifndef _TCC_H
 #define _TCC_H
 
+#include "r_types.h"
 #define _GNU_SOURCE
 #include "config.h"
 
@@ -96,7 +97,10 @@ typedef unsigned long long int uint64_t;
 #define LDOUBLE_ALIGN 4
 #define MAX_ALIGN 8
 #define PTR_SIZE 4
+
+#if !defined (__HAIKU__)
 typedef uint64_t addr_t;
+#endif
 
 /* parser debug */
 /* #define PARSE_DEBUG */
@@ -233,9 +237,9 @@ typedef struct SValue {
 typedef struct Sym {
     int v;    /* symbol token */
     char *asm_label;    /* associated asm label */
-    long r;    /* associated register */
+    unsigned int r;    /* associated register */
     union {
-        long c;    /* associated number */
+        long long c;    /* associated number */
         int *d;   /* define token stream */
     };
     CType type;    /* associated type */
@@ -249,7 +253,7 @@ typedef struct Sym {
 
 /* GNUC attribute definition */
 typedef struct AttributeDef {
-    unsigned
+    unsigned int
       func_call     : 3, /* calling convention (0..5), see below */
       aligned       : 5, /* alignement (0..16) */
       packed        : 1,
@@ -860,13 +864,13 @@ ST_FUNC void cstr_free(CString *cstr);
 ST_FUNC void cstr_reset(CString *cstr);
 
 ST_INLN void sym_free(Sym *sym);
-ST_FUNC Sym *sym_push2(Sym **ps, int v, int t, long c);
+ST_FUNC Sym *sym_push2(Sym **ps, int v, int t, long long c);
 ST_FUNC Sym *sym_find2(Sym *s, int v);
-ST_FUNC Sym *sym_push(int v, CType *type, int r, int c);
+ST_FUNC Sym *sym_push(int v, CType *type, int r, long long c);
 ST_FUNC void sym_pop(Sym **ptop, Sym *b);
 ST_INLN Sym *struct_find(int v);
 ST_INLN Sym *sym_find(int v);
-ST_FUNC Sym *global_identifier_push(int v, int t, int c);
+ST_FUNC Sym *global_identifier_push(int v, int t, long long c);
 
 ST_FUNC void tcc_open_bf(TCCState *s1, const char *filename, int initlen);
 ST_FUNC int tcc_open(TCCState *s1, const char *filename);
@@ -1022,4 +1026,6 @@ ST_FUNC int classify_x86_64_va_arg(CType *ty);
 /********************************************************/
 PUB_FUNC void tcc_appendf (const char *fmt, ...);
 
+extern void (*tcc_cb)(const char *, char **);
+
 #endif /* _TCC_H */
