@@ -20,9 +20,8 @@
 
 #include "tcc.h"
 
-/* callback pointers */
+/* callback pointer */
 ST_DATA char **tcc_cb_ptr;
-ST_DATA void (*tcc_cb)(const char *, char **);
 
 /********************************************************/
 /* global variables */
@@ -38,6 +37,9 @@ ST_DATA void **sym_pools;
 ST_DATA int nb_sym_pools;
 static int arraysize = 0;
 
+static const char *global_symname = NULL;
+static const char *global_type = NULL;
+
 ST_DATA Sym *global_stack;
 ST_DATA Sym *local_stack;
 ST_DATA Sym *scope_stack_bottom;
@@ -137,7 +139,7 @@ ST_INLN void sym_free(Sym *sym)
 }
 
 /* push, without hashing */
-ST_FUNC Sym *sym_push2(Sym **ps, int v, int t, long c)
+ST_FUNC Sym *sym_push2(Sym **ps, int v, int t, long long c)
 {
     Sym *s;
     if (ps == &local_stack) {
@@ -147,8 +149,8 @@ ST_FUNC Sym *sym_push2(Sym **ps, int v, int t, long c)
                           get_tok_str(v, NULL));
     }
 	// printf (" %d %ld set symbol '%s'\n", t, c, get_tok_str(v, NULL));
-    s = *ps;
-    s = sym_malloc();
+    // s = *ps;
+    s = sym_malloc ();
     s->asm_label = NULL;
     s->v = v;
     s->type.t = t;
@@ -195,7 +197,7 @@ ST_INLN Sym *sym_find(int v)
 }
 
 /* push a given symbol on the symbol stack */
-ST_FUNC Sym *sym_push(int v, CType *type, int r, int c)
+ST_FUNC Sym *sym_push(int v, CType *type, int r, long long c)
 {
     Sym *s, **ps;
     TokenSym *ts;
@@ -223,7 +225,7 @@ ST_FUNC Sym *sym_push(int v, CType *type, int r, int c)
 }
 
 /* push a global identifier */
-ST_FUNC Sym *global_identifier_push(int v, int t, int c)
+ST_FUNC Sym *global_identifier_push(int v, int t, long long c)
 {
     Sym *s, **ps;
     s = sym_push2(&global_stack, v, t, c);
@@ -303,7 +305,7 @@ void vpush(CType *type)
 /* push integer constant */
 ST_FUNC void vpushi(int v)
 {
-    CValue cval;
+    CValue cval = {0};
     cval.i = v;
     vsetc(&int_type, VT_CONST, &cval);
 }
@@ -390,8 +392,6 @@ static void gaddrof(void)
     /* tricky: if saved lvalue, then we can go back to lvalue */
     if ((vtop->r & VT_VALMASK) == VT_LLOCAL)
         vtop->r = (vtop->r & ~(VT_VALMASK | VT_LVAL_TYPE)) | VT_LOCAL | VT_LVAL;
-
-
 }
 
 static int pointed_size(CType *type)
@@ -400,6 +400,7 @@ static int pointed_size(CType *type)
     return type_size(pointed_type(type), &align);
 }
 
+#if 0
 static inline int is_null_pointer(SValue *p)
 {
     if ((p->r & (VT_VALMASK | VT_LVAL | VT_SYM)) != VT_CONST)
@@ -408,6 +409,7 @@ static inline int is_null_pointer(SValue *p)
         ((p->type.t & VT_BTYPE) == VT_LLONG && p->c.ll == 0) ||
 	((p->type.t & VT_BTYPE) == VT_PTR && p->c.ptr == 0);
 }
+#endif
 
 static inline int is_integer_btype(int bt)
 {
@@ -659,10 +661,14 @@ static void type_to_str(char *buf, int buf_size,
         goto no_var;
     case VT_PTR:
         s = type->ref;
-        pstrcpy(buf1, sizeof(buf1), "*");
-        if (varstr)
-            pstrcat(buf1, sizeof(buf1), varstr);
-        type_to_str(buf, buf_size, &s->type, buf1);
+        if (t & VT_ARRAY) {
+            type_to_str(buf, buf_size, &s->type, NULL);
+        } else {
+            pstrcpy(buf1, sizeof(buf1), "*");
+            if (varstr)
+                pstrcat(buf1, sizeof(buf1), varstr);
+            type_to_str(buf, buf_size, &s->type, buf1);
+        }
         goto no_var;
     }
     if (varstr) {
@@ -818,13 +824,15 @@ static void parse_attribute(AttributeDef *ad)
 static void struct_decl(CType *type, int u)
 {
     int a, v, size, align, maxalign, offset;
-	long long c;
+	long long c = 0;
     int bit_size, bit_pos, bsize, bt, lbit_pos, prevbt;
     Sym *s, *ss, *ass, **ps;
     AttributeDef ad;
     CType type1, btype;
 	const char *name = NULL;
 
+	memset (&type1, 0, sizeof (type1));
+
     a = tok; /* save decl type */
     next();
 	name = get_tok_str (tok, NULL);
@@ -857,15 +865,15 @@ static void struct_decl(CType *type, int u)
         if (s->c != -1)
             tcc_error("struct/union/enum already defined");
         /* cannot be empty */
-        c = 0;
+        c = 0LL;
         /* non empty enums are not allowed */
         if (a == TOK_ENUM) {
-			if (!strcmp (name, "{")) {
-				// UNNAMED
-				fprintf (stderr, "anonymous enums are ignored\n");
-			} else {
-				tcc_appendf ("%s=enum\n", name);
-			}
+		if (!strcmp (name, "{")) {
+			// UNNAMED
+			fprintf (stderr, "anonymous enums are ignored\n");
+		} else {
+			tcc_appendf ("%s=enum\n", name);
+		}
             for(;;) {
                 v = tok;
                 if (v < TOK_UIDENT)
@@ -875,10 +883,13 @@ static void struct_decl(CType *type, int u)
                     next();
                     c = expr_const();
                 }
-				if (strcmp (name, "{")) {
-					char *varstr = get_tok_str (v, NULL);
-					tcc_appendf ("%s.%s=%d\n", name, varstr, c);
-				}
+		if (strcmp (name, "{")) {
+			char *varstr = get_tok_str (v, NULL);
+			tcc_appendf ("%s.%s=0x%"PFMT64x"\n", name, varstr, c);
+			tcc_appendf ("%s.0x%"PFMT64x"=%s\n", name, c, varstr);
+// TODO: if token already defined throw an error
+//  if (varstr isInside (arrayOfvars)) { erprintf ("ERROR: DUP VAR IN ENUM\n"); }
+		}
                 /* enum symbols have static storage */
                 ss = sym_push(v, &llong_type, VT_CONST, c);
                 ss->type.t |= VT_STATIC;
@@ -898,7 +909,10 @@ static void struct_decl(CType *type, int u)
             bit_pos = 0;
             offset = 0;
             while (tok != '}') {
-                parse_btype(&btype, &ad);
+                if (!parse_btype(&btype, &ad)) {
+			expect("bracket");
+			break;
+		}
                 while (1) {
                     bit_size = -1;
                     v = 0;
@@ -994,12 +1008,15 @@ static void struct_decl(CType *type, int u)
 			char b[1024];
 			char *varstr = get_tok_str (v, NULL);
 			type_to_str (b, sizeof(b), &type1, NULL);
-			tcc_appendf ("%s=struct\n", name);
-			tcc_appendf ("(+)struct.%s=%s\n",
-				name, varstr);
-			/* compact form */
-			tcc_appendf ("()struct.%s.%s=%s,%d,%d\n",
-				name,varstr,b,offset,arraysize);
+			{
+				const char *ctype = (a == TOK_UNION)? "union": "struct";
+				tcc_appendf ("%s=%s\n", name, ctype);
+				tcc_appendf ("[+]%s.%s=%s\n",
+						ctype, name, varstr);
+				/* compact form */
+				tcc_appendf ("%s.%s.%s=%s,%d,%d\n",
+						ctype, name,varstr,b,offset,arraysize);
+			}
 #if 0
 			printf ("struct.%s.%s.type=%s\n", name, varstr, b);
 			printf ("struct.%s.%s.offset=%d\n", name, varstr, offset);
@@ -1260,6 +1277,8 @@ static void post_type(CType *type, AttributeDef *ad)
     Sym **plast, *s, *first;
     AttributeDef ad1;
     CType pt;
+char *symname = NULL;
+int narg  =0;
 
     if (tok == '(') {
         /* function declaration */
@@ -1267,6 +1286,14 @@ static void post_type(CType *type, AttributeDef *ad)
         l = 0;
         first = NULL;
         plast = &first;
+{
+const char *ret_type = global_type;
+free (symname);
+symname = strdup (global_symname);
+tcc_appendf ("func.%s.ret=%s\n", symname, ret_type);
+tcc_appendf ("func.%s.cc=%s\n", symname, "cdecl"); // TODO
+tcc_appendf ("%s=func\n", symname);
+}
         arg_size = 0;
         if (tok != ')') {
             for(;;) {
@@ -1297,6 +1324,13 @@ static void post_type(CType *type, AttributeDef *ad)
                 }
                 convert_parameter_type(&pt);
                 s = sym_push(n | SYM_FIELD, &pt, 0, 0);
+{
+	    char kind[1024];
+	    type_to_str (kind, sizeof (kind), &pt, NULL);
+	    tcc_appendf ("func.%s.arg.%d=%s,%s\n",
+		symname, narg, kind, global_symname);
+	    narg++;
+}
                 *plast = s;
                 plast = &s->next;
                 if (tok == ')')
@@ -1309,6 +1343,7 @@ static void post_type(CType *type, AttributeDef *ad)
                 }
             }
         }
+	tcc_appendf ("func.%s.args=%d\n", symname, narg);
         /* if no parameters, then old type prototype */
         if (l == 0)
             l = FUNC_OLD;
@@ -1330,6 +1365,8 @@ static void post_type(CType *type, AttributeDef *ad)
         s->next = first;
         type->t = VT_FUNC;
         type->ref = s;
+free(symname);
+symname = NULL;
     } else if (tok == '[') {
         /* array definition */
         next();
@@ -1438,20 +1475,30 @@ static void type_decl(CType *type, AttributeDef *ad, int *v, int td)
     if (storage & VT_STATIC) {
         int saved_nocode_wanted = nocode_wanted;
         nocode_wanted = 1;
+//eprintf ("STATIC %s\n", get_tok_str(*v, NULL));
         post_type(type, ad);
         nocode_wanted = saved_nocode_wanted;
-    } else
-        post_type(type, ad);
+    } else {
+	    static char kind[1024];
+	    char *name = get_tok_str (*v, NULL);
+	    type_to_str (kind, sizeof(kind), type, NULL);
+	    //eprintf ("---%d %s STATIC %s\n", td, kind, name);
+	    global_symname = name;
+	    global_type = kind;
+	    post_type(type, ad);
+    }
     type->t |= storage;
     if (tok == TOK_ATTRIBUTE1 || tok == TOK_ATTRIBUTE2)
         parse_attribute(ad);
 
-    if (!type1.t)
+    if (!type1.t) {
         return;
+    }
     /* append type at the end of type1 */
     type2 = &type1;
     for(;;) {
         s = type2->ref;
+//eprintf ("ARG:\n");
         type2 = &s->type;
         if (!type2->t) {
             *type2 = *type;
@@ -1587,7 +1634,7 @@ ST_FUNC void unary(void)
         /* fall thru */
     case TOK___FUNC__:
         {
-            void *ptr = NULL;
+            // void *ptr = NULL;
             int len;
             /* special function name identifier */
             len = strlen(funcname) + 1;
@@ -1596,7 +1643,8 @@ ST_FUNC void unary(void)
             mk_pointer(&type);
             type.t |= VT_ARRAY;
             type.ref->c = len;
-            memcpy(ptr, funcname, len);
+            // XXX ptr is NULL HERE WTF
+            // memcpy(ptr, funcname, len);
             next();
         }
         break;
@@ -1790,6 +1838,9 @@ ST_FUNC void unary(void)
             if (tok != '(')
                 tcc_error("'%s' undeclared", get_tok_str(t, NULL));
         }
+        if (!s) {
+           tcc_error("invalid declaration '%s'", get_tok_str(t, NULL));
+        }
         if ((s->type.t & (VT_STATIC | VT_INLINE | VT_BTYPE)) ==
             (VT_STATIC | VT_INLINE | VT_FUNC)) {
             /* if referencing an inline function, then we generate a
@@ -1834,7 +1885,9 @@ ST_FUNC void unary(void)
                 tcc_error("field not found: %s",  get_tok_str(tok & ~SYM_FIELD, NULL));
             /* add field offset to pointer */
             vtop->type = char_pointer_type; /* change type to 'char *' */
+if (s) {
             vpushi(s->c);
+}
             /* change type to field type, and set to lvalue */
             vtop->type = s->type;
             vtop->type.t |= qualifiers;
@@ -2075,7 +2128,7 @@ static void expr_const1(void)
 /* parse an integer constant and return its value. */
 ST_FUNC long long expr_const(void)
 {
-    long long c;
+    long long c = 0LL;
     expr_const1();
     if ((vtop->r & (VT_VALMASK | VT_LVAL | VT_SYM)) != VT_CONST)
         expect("constant expression");
@@ -2112,14 +2165,12 @@ static void decl_designator(CType *type, unsigned long c,
                             long long *cur_index, Sym **cur_field,
                             int size_only)
 {
-    Sym *s, *f;
-	long long index, index_last;
+    Sym *s, *f = NULL;
+    long long index, index_last;
     int notfirst, align, l, nb_elems, elem_size;
     CType type1;
 
     notfirst = 0;
-    elem_size = 0;
-    nb_elems = 1;
     if (gnu_ext && (l = is_label()) != 0)
         goto struct_field;
     while (tok == '[' || tok == '.') {
@@ -2173,10 +2224,12 @@ static void decl_designator(CType *type, unsigned long c,
             if (!notfirst)
                 *cur_field = f;
             /* XXX: fix this mess by using explicit storage field */
+if (f) {
             type1 = f->type;
             type1.t |= (type->t & ~VT_TYPE);
             type = &type1;
             c += f->c;
+}
         }
         notfirst = 1;
     }
@@ -2197,10 +2250,12 @@ static void decl_designator(CType *type, unsigned long c,
             if (!f)
                 tcc_error("too many field init");
             /* XXX: fix this mess by using explicit storage field */
+if (f) {
             type1 = f->type;
             type1.t |= (type->t & ~VT_TYPE);
             type = &type1;
             c += f->c;
+}
         }
     }
     decl_initializer(type, c, 0, size_only);
@@ -2259,7 +2314,8 @@ static void init_putz(CType *t, unsigned long c, int size)
 static void decl_initializer(CType *type, unsigned long c,
                              int first, int size_only)
 {
-    int index, array_length, n, no_oblock, nb, parlevel, parlevel1, i;
+    long long index;
+    int array_length, n, no_oblock, nb, parlevel, parlevel1, i;
     int size1, align1, expr_type;
     Sym *s, *f;
     CType *t1;
@@ -2626,7 +2682,7 @@ static void decl_initializer_alloc(CType *type, AttributeDef *ad, int r,
             vtop->sym = sym;
         }
         /* patch symbol weakness */
-        if (type->t & VT_WEAK)
+        if ((type->t & VT_WEAK) && sym)
             weaken_symbol(sym);
     }
     no_alloc: ;
@@ -2638,7 +2694,7 @@ static void func_decl_list(Sym *func_sym)
 {
     AttributeDef ad;
     int v;
-    Sym *s;
+    Sym *s = NULL;
     CType btype, type;
 
     /* parse each declaration */
@@ -2669,7 +2725,9 @@ static void func_decl_list(Sym *func_sym)
                     tcc_error("storage class specified for '%s'", get_tok_str(v, NULL));
                 convert_parameter_type(&type);
                 /* we can add the type (NOTE: it could be local to the function) */
+if (s) {
                 s->type = type;
+}
                 /* accept other parameters */
                 if (tok == ',')
                     next();
@@ -2703,7 +2761,7 @@ static int decl0(int l, int is_for_loop_init)
                 (tok == TOK_ASM1 || tok == TOK_ASM2 || tok == TOK_ASM3)) {
                 /* global asm block */
 #if 1
-printf ("global asm not supported\n");
+eprintf ("global asm not supported\n");
 return 1;
 #endif
                 //asm_global_instr();
